@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalgrid.cpp 15580 2008-10-22 07:33:59Z dron $
+ * $Id: gdalgrid.cpp 15727 2008-11-13 19:29:57Z warmerdam $
  *
  * Project:  GDAL Gridding API.
  * Purpose:  Implementation of GDAL scattered data gridder.
@@ -29,7 +29,7 @@
 
 #include "gdalgrid.h"
 
-CPL_CVSID("$Id: gdalgrid.cpp 15580 2008-10-22 07:33:59Z dron $");
+CPL_CVSID("$Id: gdalgrid.cpp 15727 2008-11-13 19:29:57Z warmerdam $");
 
 #define TO_RADIANS (3.14159265358979323846 / 180.0)
 
@@ -55,9 +55,10 @@ CPL_CVSID("$Id: gdalgrid.cpp 15580 2008-10-22 07:33:59Z dron $");
  *
  *  where 
  *  <ul>
- *      <li> \f$r\f$ is a distance from the grid node to point \f$i\f$,
  *      <li> \f$Z_i\f$ is a known value at point \f$i\f$,
- *      <li> \f$p\f$ is a weighting power.
+ *      <li> \f$r\f$ is a distance from the grid node to point \f$i\f$,
+ *      <li> \f$p\f$ is a weighting power,
+ *      <li> \f$n\f$ is a number of points in search ellipse.
  *  </ul>
  *
  *  In this method the weighting factor \f$w\f$ is
@@ -69,9 +70,9 @@ CPL_CVSID("$Id: gdalgrid.cpp 15580 2008-10-22 07:33:59Z dron $");
  * @param poOptions Algorithm parameters. This should point to
  * GDALGridInverseDistanceToAPowerOptions object. 
  * @param nPoints Number of elements in input arrays.
- * @param pdfX Input array of X coordinates. 
- * @param pdfY Input array of Y coordinates. 
- * @param pdfZ Input array of Z values. 
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
  * @param dfXPoint X coordinate of the point to compute.
  * @param dfYPoint Y coordinate of the point to compute.
  * @param nXPoint X position of the point to compute.  
@@ -84,8 +85,8 @@ CPL_CVSID("$Id: gdalgrid.cpp 15580 2008-10-22 07:33:59Z dron $");
 
 CPLErr
 GDALGridInverseDistanceToAPower( const void *poOptions, GUInt32 nPoints,
-                                 const double *pdfX, const double *pdfY,
-                                 const double *pdfZ,
+                                 const double *padfX, const double *padfY,
+                                 const double *padfZ,
                                  double dfXPoint, double dfYPoint,
                                  double *pdfValue )
 {
@@ -125,8 +126,8 @@ GDALGridInverseDistanceToAPower( const void *poOptions, GUInt32 nPoints,
 
     for ( i = 0; i < nPoints; i++ )
     {
-        double  dfRX = pdfX[i] - dfXPoint;
-        double  dfRY = pdfY[i] - dfYPoint;
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
         const double dfR2 =
             dfRX * dfRX + dfRY * dfRY + dfSmoothing * dfSmoothing;
 
@@ -144,13 +145,13 @@ GDALGridInverseDistanceToAPower( const void *poOptions, GUInt32 nPoints,
         {
             if ( CPLIsEqual(dfR2, 0.0) )
             {
-                (*pdfValue) = pdfZ[i];
+                (*pdfValue) = padfZ[i];
                 return CE_None;
             }
             else
             {
                 const double  dfW = pow( sqrt(dfR2), dfPower );
-                dfNominator += pdfZ[i] / dfW;
+                dfNominator += padfZ[i] / dfW;
                 dfDenominator += 1.0 / dfW;
                 n++;
                 if ( nMaxPoints > 0 && n > nMaxPoints )
@@ -188,8 +189,8 @@ GDALGridInverseDistanceToAPower( const void *poOptions, GUInt32 nPoints,
 
 CPLErr
 GDALGridInverseDistanceToAPowerNoSearch( const void *poOptions, GUInt32 nPoints,
-                                         const double *pdfX, const double *pdfY,
-                                         const double *pdfZ,
+                                         const double *padfX, const double *padfY,
+                                         const double *padfZ,
                                          double dfXPoint, double dfYPoint,
                                          double *pdfValue )
 {
@@ -202,20 +203,20 @@ GDALGridInverseDistanceToAPowerNoSearch( const void *poOptions, GUInt32 nPoints,
 
     for ( i = 0; i < nPoints; i++ )
     {
-        const double dfRX = pdfX[i] - dfXPoint;
-        const double dfRY = pdfY[i] - dfYPoint;
+        const double dfRX = padfX[i] - dfXPoint;
+        const double dfRY = padfY[i] - dfYPoint;
         const double dfR2 =
             dfRX * dfRX + dfRY * dfRY + dfSmoothing * dfSmoothing;
 
         if ( CPLIsEqual(dfR2, 0.0) )
         {
-            (*pdfValue) = pdfZ[i];
+            (*pdfValue) = padfZ[i];
             return CE_None;
         }
         else
         {
             const double dfW = pow( sqrt(dfR2), dfPower );
-            dfNominator += pdfZ[i] / dfW;
+            dfNominator += padfZ[i] / dfW;
             dfDenominator += 1.0 / dfW;
         }
     }
@@ -244,12 +245,25 @@ GDALGridInverseDistanceToAPowerNoSearch( const void *poOptions, GUInt32 nPoints,
  * points to average can be set, if there are not enough points in window, the
  * grid node considered empty and will be filled with specified NODATA value.
  *
+ * Mathematically it can be expressed with the formula:
+ *
+ * \f[
+ *      Z=\frac{\sum_{i=1}^n{Z_i}}{n}
+ * \f]
+ *
+ *  where 
+ *  <ul>
+ *      <li> \f$Z\f$ is a resulting value at the grid node,
+ *      <li> \f$Z_i\f$ is a known value at point \f$i\f$,
+ *      <li> \f$n\f$ is a number of points in search ellipse.
+ *  </ul>
+ *
  * @param poOptions Algorithm parameters. This should point to
  * GDALGridMovingAverageOptions object. 
  * @param nPoints Number of elements in input arrays.
- * @param pdfX Input array of X coordinates. 
- * @param pdfY Input array of Y coordinates. 
- * @param pdfZ Input array of Z values. 
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
  * @param dfXPoint X coordinate of the point to compute.
  * @param dfYPoint Y coordinate of the point to compute.
  * @param pdfValue Pointer to variable where the computed grid node value
@@ -260,8 +274,8 @@ GDALGridInverseDistanceToAPowerNoSearch( const void *poOptions, GUInt32 nPoints,
 
 CPLErr
 GDALGridMovingAverage( const void *poOptions, GUInt32 nPoints,
-                       const double *pdfX, const double *pdfY,
-                       const double *pdfZ,
+                       const double *padfX, const double *padfY,
+                       const double *padfZ,
                        double dfXPoint, double dfYPoint, double *pdfValue )
 {
     // TODO: For optimization purposes pre-computed parameters should be moved
@@ -292,8 +306,8 @@ GDALGridMovingAverage( const void *poOptions, GUInt32 nPoints,
 
     while ( i < nPoints )
     {
-        double  dfRX = pdfX[i] - dfXPoint;
-        double  dfRY = pdfY[i] - dfYPoint;
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
 
         if ( bRotated )
         {
@@ -307,7 +321,7 @@ GDALGridMovingAverage( const void *poOptions, GUInt32 nPoints,
         // Is this point located inside the search ellipse?
         if ( dfRadius2 * dfRX * dfRX + dfRadius1 * dfRY * dfRY <= dfR12 )
         {
-            dfAccumulator += pdfZ[i];
+            dfAccumulator += padfZ[i];
             n++;
         }
 
@@ -341,9 +355,9 @@ GDALGridMovingAverage( const void *poOptions, GUInt32 nPoints,
  * @param poOptions Algorithm parameters. This should point to
  * GDALGridNearestNeighborOptions object. 
  * @param nPoints Number of elements in input arrays.
- * @param pdfX Input array of X coordinates. 
- * @param pdfY Input array of Y coordinates. 
- * @param pdfZ Input array of Z values. 
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
  * @param dfXPoint X coordinate of the point to compute.
  * @param dfYPoint Y coordinate of the point to compute.
  * @param pdfValue Pointer to variable where the computed grid node value
@@ -354,8 +368,8 @@ GDALGridMovingAverage( const void *poOptions, GUInt32 nPoints,
 
 CPLErr
 GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
-                         const double *pdfX, const double *pdfY,
-                         const double *pdfZ,
+                         const double *padfX, const double *padfY,
+                         const double *padfZ,
                          double dfXPoint, double dfYPoint, double *pdfValue )
 {
     // TODO: For optimization purposes pre-computed parameters should be moved
@@ -393,8 +407,8 @@ GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
 
     while ( i < nPoints )
     {
-        double  dfRX = pdfX[i] - dfXPoint;
-        double  dfRY = pdfY[i] - dfYPoint;
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
 
         if ( bRotated )
         {
@@ -412,7 +426,7 @@ GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
             if ( dfNearestR == 0.0 || dfR2 < dfNearestR )
             {
                 dfNearestR = dfR2;
-                dfNearestValue = pdfZ[i];
+                dfNearestValue = padfZ[i];
             }
         }
 
@@ -424,6 +438,342 @@ GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
     return CE_None;
 }
 
+/************************************************************************/
+/*                      GDALGridDataMetricMinimum()                     */
+/************************************************************************/
+
+/**
+ * Minimum data value (data metric).
+ *
+ * Minimum value found in grid node search ellipse. If there are no points
+ * found, the specified NODATA value will be returned.
+ *
+ * \f[
+ *      Z=\min{(Z_1,Z_2,\ldots,Z_n)}
+ * \f]
+ *
+ *  where 
+ *  <ul>
+ *      <li> \f$Z\f$ is a resulting value at the grid node,
+ *      <li> \f$Z_i\f$ is a known value at point \f$i\f$,
+ *      <li> \f$n\f$ is a number of points in search ellipse.
+ *  </ul>
+ *
+ * @param poOptions Algorithm parameters. This should point to
+ * GDALGridDataMetricsOptions object. 
+ * @param nPoints Number of elements in input arrays.
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
+ * @param dfXPoint X coordinate of the point to compute.
+ * @param dfYPoint Y coordinate of the point to compute.
+ * @param pdfValue Pointer to variable where the computed grid node value
+ * will be returned.
+ *
+ * @return CE_None on success or CE_Failure if something goes wrong.
+ */
+
+CPLErr
+GDALGridDataMetricMinimum( const void *poOptions, GUInt32 nPoints,
+                           const double *padfX, const double *padfY,
+                           const double *padfZ,
+                           double dfXPoint, double dfYPoint, double *pdfValue )
+{
+    // TODO: For optimization purposes pre-computed parameters should be moved
+    // out of this routine to the calling function.
+
+    // Pre-compute search ellipse parameters
+    double  dfRadius1 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius1;
+    double  dfRadius2 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius2;
+    double  dfR12;
+
+    dfRadius1 *= dfRadius1;
+    dfRadius2 *= dfRadius2;
+    dfR12 = dfRadius1 * dfRadius2;
+
+    // Compute coefficients for coordinate system rotation.
+    double      dfCoeff1 = 0.0, dfCoeff2 = 0.0;
+    const double dfAngle =
+        TO_RADIANS * ((GDALGridNearestNeighborOptions *)poOptions)->dfAngle;
+    const bool  bRotated = ( dfAngle == 0.0 ) ? false : true;
+    if ( bRotated )
+    {
+        dfCoeff1 = cos(dfAngle);
+        dfCoeff2 = sin(dfAngle);
+    }
+
+    double      dfMinimumValue=0.0;
+    GUInt32     i = 0, n = 0;
+
+    while ( i < nPoints )
+    {
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
+
+        if ( bRotated )
+        {
+            double dfRXRotated = dfRX * dfCoeff1 + dfRY * dfCoeff2;
+            double dfRYRotated = dfRY * dfCoeff1 - dfRX * dfCoeff2;
+
+            dfRX = dfRXRotated;
+            dfRY = dfRYRotated;
+        }
+
+        // Is this point located inside the search ellipse?
+        if ( dfRadius2 * dfRX * dfRX + dfRadius1 * dfRY * dfRY <= dfR12 )
+        {
+            if ( n )
+            {
+                if ( dfMinimumValue > padfZ[i] )
+                    dfMinimumValue = padfZ[i];
+            }
+            else
+                dfMinimumValue = padfZ[i];
+            n++;
+        }
+
+        i++;
+    }
+
+    if ( n < ((GDALGridMovingAverageOptions *)poOptions)->nMinPoints
+         || n == 0 )
+    {
+        (*pdfValue) =
+            ((GDALGridMovingAverageOptions *)poOptions)->dfNoDataValue;
+    }
+    else
+        (*pdfValue) = dfMinimumValue;
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                      GDALGridDataMetricMaximum()                     */
+/************************************************************************/
+
+/**
+ * Maximum data value (data metric).
+ *
+ * Maximum value found in grid node search ellipse. If there are no points
+ * found, the specified NODATA value will be returned.
+ *
+ * \f[
+ *      Z=\max{(Z_1,Z_2,\ldots,Z_n)}
+ * \f]
+ *
+ *  where 
+ *  <ul>
+ *      <li> \f$Z\f$ is a resulting value at the grid node,
+ *      <li> \f$Z_i\f$ is a known value at point \f$i\f$,
+ *      <li> \f$n\f$ is a number of points in search ellipse.
+ *  </ul>
+ *
+ * @param poOptions Algorithm parameters. This should point to
+ * GDALGridDataMetricsOptions object. 
+ * @param nPoints Number of elements in input arrays.
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
+ * @param dfXPoint X coordinate of the point to compute.
+ * @param dfYPoint Y coordinate of the point to compute.
+ * @param pdfValue Pointer to variable where the computed grid node value
+ * will be returned.
+ *
+ * @return CE_None on success or CE_Failure if something goes wrong.
+ */
+
+CPLErr
+GDALGridDataMetricMaximum( const void *poOptions, GUInt32 nPoints,
+                           const double *padfX, const double *padfY,
+                           const double *padfZ,
+                           double dfXPoint, double dfYPoint, double *pdfValue )
+{
+    // TODO: For optimization purposes pre-computed parameters should be moved
+    // out of this routine to the calling function.
+
+    // Pre-compute search ellipse parameters
+    double  dfRadius1 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius1;
+    double  dfRadius2 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius2;
+    double  dfR12;
+
+    dfRadius1 *= dfRadius1;
+    dfRadius2 *= dfRadius2;
+    dfR12 = dfRadius1 * dfRadius2;
+
+    // Compute coefficients for coordinate system rotation.
+    double      dfCoeff1 = 0.0, dfCoeff2 = 0.0;
+    const double    dfAngle =
+        TO_RADIANS * ((GDALGridNearestNeighborOptions *)poOptions)->dfAngle;
+    const bool  bRotated = ( dfAngle == 0.0 ) ? false : true;
+    if ( bRotated )
+    {
+        dfCoeff1 = cos(dfAngle);
+        dfCoeff2 = sin(dfAngle);
+    }
+
+    double      dfMaximumValue=0.0;
+    GUInt32     i = 0, n = 0;
+
+    while ( i < nPoints )
+    {
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
+
+        if ( bRotated )
+        {
+            double dfRXRotated = dfRX * dfCoeff1 + dfRY * dfCoeff2;
+            double dfRYRotated = dfRY * dfCoeff1 - dfRX * dfCoeff2;
+
+            dfRX = dfRXRotated;
+            dfRY = dfRYRotated;
+        }
+
+        // Is this point located inside the search ellipse?
+        if ( dfRadius2 * dfRX * dfRX + dfRadius1 * dfRY * dfRY <= dfR12 )
+        {
+            if ( n )
+            {
+                if ( dfMaximumValue < padfZ[i] )
+                    dfMaximumValue = padfZ[i];
+            }
+            else
+                dfMaximumValue = padfZ[i];
+            n++;
+        }
+
+        i++;
+    }
+
+    if ( n < ((GDALGridMovingAverageOptions *)poOptions)->nMinPoints
+         || n == 0 )
+    {
+        (*pdfValue) =
+            ((GDALGridMovingAverageOptions *)poOptions)->dfNoDataValue;
+    }
+    else
+        (*pdfValue) = dfMaximumValue;
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                       GDALGridDataMetricRange()                      */
+/************************************************************************/
+
+/**
+ * Data range (data metric).
+ *
+ * A difference between the minimum and maximum values found in grid node
+ * search ellipse. If there are no points found, the specified NODATA
+ * value will be returned.
+ *
+ * \f[
+ *      Z=\max{(Z_1,Z_2,\ldots,Z_n)}-\min{(Z_1,Z_2,\ldots,Z_n)}
+ * \f]
+ *
+ *  where 
+ *  <ul>
+ *      <li> \f$Z\f$ is a resulting value at the grid node,
+ *      <li> \f$Z_i\f$ is a known value at point \f$i\f$,
+ *      <li> \f$n\f$ is a number of points in search ellipse.
+ *  </ul>
+ *
+ * @param poOptions Algorithm parameters. This should point to
+ * GDALGridDataMetricsOptions object. 
+ * @param nPoints Number of elements in input arrays.
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
+ * @param dfXPoint X coordinate of the point to compute.
+ * @param dfYPoint Y coordinate of the point to compute.
+ * @param pdfValue Pointer to variable where the computed grid node value
+ * will be returned.
+ *
+ * @return CE_None on success or CE_Failure if something goes wrong.
+ */
+
+CPLErr
+GDALGridDataMetricRange( const void *poOptions, GUInt32 nPoints,
+                         const double *padfX, const double *padfY,
+                         const double *padfZ,
+                         double dfXPoint, double dfYPoint, double *pdfValue )
+{
+    // TODO: For optimization purposes pre-computed parameters should be moved
+    // out of this routine to the calling function.
+
+    // Pre-compute search ellipse parameters
+    double  dfRadius1 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius1;
+    double  dfRadius2 =
+        ((GDALGridNearestNeighborOptions *)poOptions)->dfRadius2;
+    double  dfR12;
+
+    dfRadius1 *= dfRadius1;
+    dfRadius2 *= dfRadius2;
+    dfR12 = dfRadius1 * dfRadius2;
+
+    // Compute coefficients for coordinate system rotation.
+    double      dfCoeff1 = 0.0, dfCoeff2 = 0.0;
+    const double    dfAngle =
+        TO_RADIANS * ((GDALGridNearestNeighborOptions *)poOptions)->dfAngle;
+    const bool  bRotated = ( dfAngle == 0.0 ) ? false : true;
+    if ( bRotated )
+    {
+        dfCoeff1 = cos(dfAngle);
+        dfCoeff2 = sin(dfAngle);
+    }
+
+    double      dfMaximumValue=0.0, dfMinimumValue=0.0;
+    GUInt32     i = 0, n = 0;
+
+    while ( i < nPoints )
+    {
+        double  dfRX = padfX[i] - dfXPoint;
+        double  dfRY = padfY[i] - dfYPoint;
+
+        if ( bRotated )
+        {
+            double dfRXRotated = dfRX * dfCoeff1 + dfRY * dfCoeff2;
+            double dfRYRotated = dfRY * dfCoeff1 - dfRX * dfCoeff2;
+
+            dfRX = dfRXRotated;
+            dfRY = dfRYRotated;
+        }
+
+        // Is this point located inside the search ellipse?
+        if ( dfRadius2 * dfRX * dfRX + dfRadius1 * dfRY * dfRY <= dfR12 )
+        {
+            if ( n )
+            {
+                if ( dfMinimumValue > padfZ[i] )
+                    dfMinimumValue = padfZ[i];
+                if ( dfMaximumValue < padfZ[i] )
+                    dfMaximumValue = padfZ[i];
+            }
+            else
+                dfMinimumValue = dfMaximumValue = padfZ[i];
+            n++;
+        }
+
+        i++;
+    }
+
+    if ( n < ((GDALGridMovingAverageOptions *)poOptions)->nMinPoints
+         || n == 0 )
+    {
+        (*pdfValue) =
+            ((GDALGridMovingAverageOptions *)poOptions)->dfNoDataValue;
+    }
+    else
+        (*pdfValue) = dfMaximumValue - dfMinimumValue;
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                            GDALGridCreate()                          */
 /************************************************************************/
@@ -439,9 +789,9 @@ GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
  * @param eAlgorithm Gridding method. 
  * @param poOptions Options to control choosen gridding method.
  * @param nPoints Number of elements in input arrays.
- * @param pdfX Input array of X coordinates. 
- * @param pdfY Input array of Y coordinates. 
- * @param pdfZ Input array of Z values. 
+ * @param padfX Input array of X coordinates. 
+ * @param padfY Input array of Y coordinates. 
+ * @param padfZ Input array of Z values. 
  * @param dfXMin Lowest X border of output grid.
  * @param dfXMax Highest X border of output grid.
  * @param dfYMin Lowest Y border of output grid.
@@ -460,15 +810,15 @@ GDALGridNearestNeighbor( const void *poOptions, GUInt32 nPoints,
 CPLErr
 GDALGridCreate( GDALGridAlgorithm eAlgorithm, const void *poOptions,
                 GUInt32 nPoints,
-                const double *pdfX, const double *pdfY, const double *pdfZ,
+                const double *padfX, const double *padfY, const double *padfZ,
                 double dfXMin, double dfXMax, double dfYMin, double dfYMax,
                 GUInt32 nXSize, GUInt32 nYSize, GDALDataType eType, void *pData,
                 GDALProgressFunc pfnProgress, void *pProgressArg )
 {
     CPLAssert( poOptions );
-    CPLAssert( pdfX );
-    CPLAssert( pdfY );
-    CPLAssert( pdfZ );
+    CPLAssert( padfX );
+    CPLAssert( padfY );
+    CPLAssert( padfZ );
     CPLAssert( pData );
 
     if ( nXSize == 0 || nYSize == 0 )
@@ -500,6 +850,18 @@ GDALGridCreate( GDALGridAlgorithm eAlgorithm, const void *poOptions,
             pfnGDALGridMethod = GDALGridNearestNeighbor;
             break;
 
+        case GGA_MetricMinimum:
+            pfnGDALGridMethod = GDALGridDataMetricMinimum;
+            break;
+
+        case GGA_MetricMaximum:
+            pfnGDALGridMethod = GDALGridDataMetricMaximum;
+            break;
+
+        case GGA_MetricRange:
+            pfnGDALGridMethod = GDALGridDataMetricRange;
+            break;
+
         default:
             CPLError( CE_Failure, CPLE_IllegalArg,
                       "GDAL does not support gridding method %d", eAlgorithm );
@@ -517,13 +879,14 @@ GDALGridCreate( GDALGridAlgorithm eAlgorithm, const void *poOptions,
         {
             const double    dfXPoint = dfXMin + ( nXPoint + 0.5 ) * dfDeltaX;
             double          dfValue = 0.0;
-            if ( (*pfnGDALGridMethod)( poOptions, nPoints, pdfX, pdfY, pdfZ,
+            if ( (*pfnGDALGridMethod)( poOptions, nPoints, padfX, padfY, padfZ,
                                        dfXPoint, dfYPoint,
                                        &dfValue ) != CE_None )
             {
                 CPLError( CE_Failure, CPLE_AppDefined,
                           "Gridding failed at X position %lu, Y position %lu",
-                          nXPoint, nYPoint );
+                          (long unsigned int)nXPoint,
+                          (long unsigned int)nYPoint );
                 return CE_Failure;
             }
 
