@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaladdo.cpp 12899 2007-11-20 21:56:33Z warmerdam $
+ * $Id: gdaladdo.cpp 15414 2008-09-22 21:00:49Z rouault $
  *
  * Project:  GDAL Utilities
  * Purpose:  Commandline application to build overviews. 
@@ -30,7 +30,7 @@
 #include "gdal_priv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdaladdo.cpp 12899 2007-11-20 21:56:33Z warmerdam $");
+CPL_CVSID("$Id: gdaladdo.cpp 15414 2008-09-22 21:00:49Z rouault $");
 
 /************************************************************************/
 /*                               Usage()                                */
@@ -39,11 +39,24 @@ CPL_CVSID("$Id: gdaladdo.cpp 12899 2007-11-20 21:56:33Z warmerdam $");
 static void Usage()
 
 {
-    printf( "Usage: gdaladdo [-r {nearest,average,average_mp,average_magphase,mode}]\n"
-            "                [--help-general] filename levels\n"
+    printf( "Usage: gdaladdo [-r {nearest,average,gauss,average_mp,average_magphase,mode}]\n"
+            "                [-ro] [--help-general] filename levels\n"
             "\n"
-            "Example:\n"
-            " %% gdaladdo -r average abc.tif 2 4 8 16\n" );
+            "  -r : choice of resampling method\n"
+            "  -ro : open the dataset in read-only mode, in order to generate\n"
+            "        external overview (for GeoTIFF datasets especially)\n"
+            "\n"
+            "Usefull configuration variables :\n"
+            "  --config USE_RRD YES : Use Erdas Imagine format (.aux) as overview format.\n"
+            "Below, only for external overviews in GeoTIFF format:\n"
+            "  --config COMPRESS_OVERVIEW {JPEG,LZW,PACKBITS,DEFLATE} : TIFF compression method\n"
+            "  --config PHOTOMETRIC_OVERVIEW {RGB,YCBCR,...} : TIFF photometric interpretation\n"
+            "  --config INTERLEAVE_OVERVIEW {PIXEL|BAND} : TIFF interleaving method\n"
+            "\n"
+            "Examples:\n"
+            " %% gdaladdo -r average abc.tif 2 4 8 16\n"
+            " %% gdaladdo --config COMPRESS_OVERVIEW JPEG --config PHOTOMETRIC_OVERVIEW YCBCR\n"
+            "             --config INTERLEAVE_OVERVIEW PIXEL -ro abc.tif 2 4 8 16\n");
     exit( 1 );
 }
 
@@ -54,12 +67,22 @@ static void Usage()
 int main( int nArgc, char ** papszArgv )
 
 {
-    GDALDataset *poDataset;
+    GDALDatasetH hDataset;
     const char * pszResampling = "nearest";
     const char * pszFilename = NULL;
     int          anLevels[1024];
     int          nLevelCount = 0;
     int          nResultStatus = 0;
+    int          bReadOnly = FALSE;
+
+    /* Check that we are running against at least GDAL 1.4 */
+    /* Note to developers : if we use newer API, please change the requirement */
+    if (atoi(GDALVersionInfo("VERSION_NUM")) < 1400)
+    {
+        fprintf(stderr, "At least, GDAL >= 1.4.0 is required for this version of %s, "
+                        "which was compiled against GDAL %s\n", papszArgv[0], GDAL_RELEASE_NAME);
+        exit(1);
+    }
 
     GDALAllRegister();
 
@@ -72,8 +95,16 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
     for( int iArg = 1; iArg < nArgc; iArg++ )
     {
-        if( EQUAL(papszArgv[iArg],"-r") && iArg < nArgc-1 )
+        if( EQUAL(papszArgv[iArg], "--utility_version") )
+        {
+            printf("%s was compiled against GDAL %s and is running against GDAL %s\n",
+                   papszArgv[0], GDAL_RELEASE_NAME, GDALVersionInfo("RELEASE_NAME"));
+            return 0;
+        }
+        else if( EQUAL(papszArgv[iArg],"-r") && iArg < nArgc-1 )
             pszResampling = papszArgv[++iArg];
+        else if( EQUAL(papszArgv[iArg],"-ro"))
+            bReadOnly = TRUE;
         else if( pszFilename == NULL )
             pszFilename = papszArgv[iArg];
         else if( atoi(papszArgv[iArg]) > 0 )
@@ -88,20 +119,22 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
 /*      Open data file.                                                 */
 /* -------------------------------------------------------------------- */
-    poDataset = (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+    if (bReadOnly)
+        hDataset = NULL;
+    else
+        hDataset = GDALOpen( pszFilename, GA_Update );
 
-    if( poDataset == NULL )
-        poDataset = (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+    if( hDataset == NULL )
+        hDataset = GDALOpen( pszFilename, GA_ReadOnly );
 
-    if( poDataset == NULL )
+    if( hDataset == NULL )
         exit( 2 );
 
 /* -------------------------------------------------------------------- */
 /*      Generate overviews.                                             */
 /* -------------------------------------------------------------------- */
-    if( poDataset->BuildOverviews( pszResampling, 
-                                   nLevelCount, anLevels, 0, NULL,
-                                   GDALTermProgress, NULL ) != CE_None )
+    if (GDALBuildOverviews( hDataset,pszResampling, nLevelCount, anLevels,
+                             0, NULL, GDALTermProgress, NULL ) != CE_None )
     {
         printf( "Overview building failed.\n" );
         nResultStatus = 100;
@@ -110,7 +143,7 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
 /*      Cleanup                                                         */
 /* -------------------------------------------------------------------- */
-    delete poDataset;
+    GDALClose(hDataset);
 
     CSLDestroy( papszArgv );
     GDALDestroyDriverManager();
