@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaltindex.c 15960 2008-12-15 18:24:08Z rouault $
+ * $Id: gdaltindex.c 14822 2008-07-05 11:19:31Z rouault $
  *
  * Project:  MapServer
  * Purpose:  Commandline App to build tile index for raster files.
@@ -32,7 +32,7 @@
 #include "cpl_port.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: gdaltindex.c 15960 2008-12-15 18:24:08Z rouault $");
+CPL_CVSID("$Id: gdaltindex.c 14822 2008-07-05 11:19:31Z rouault $");
 
 /************************************************************************/
 /*                               Usage()                                */
@@ -81,6 +81,15 @@ int main(int argc, char *argv[])
     int alreadyExistingProjectionRefValid = FALSE;
     char* alreadyExistingProjectionRef = NULL;
 
+    /* Check that we are running against at least GDAL 1.4 */
+    /* Note to developers : if we use newer API, please change the requirement */
+    if (atoi(GDALVersionInfo("VERSION_NUM")) < 1400)
+    {
+        fprintf(stderr, "At least, GDAL >= 1.4.0 is required for this version of %s, "
+                "which was compiled against GDAL %s\n", argv[0], GDAL_RELEASE_NAME);
+        exit(1);
+    }
+
     GDALAllRegister();
 
     argc = GDALGeneralCmdLineProcessor( argc, &argv, 0 );
@@ -92,7 +101,13 @@ int main(int argc, char *argv[])
 /* -------------------------------------------------------------------- */
     for( i_arg = 1; i_arg < argc; i_arg++ )
     {
-        if( strcmp(argv[i_arg],"-tileindex") == 0 )
+        if( EQUAL(argv[i_arg], "--utility_version") )
+        {
+            printf("%s was compiled against GDAL %s and is running against GDAL %s\n",
+                   argv[0], GDAL_RELEASE_NAME, GDALVersionInfo("RELEASE_NAME"));
+            return 0;
+        }
+        else if( strcmp(argv[i_arg],"-tileindex") == 0 )
         {
             tile_index = argv[++i_arg];
         }
@@ -233,7 +248,6 @@ int main(int argc, char *argv[])
         }
         if (i != nExistingFiles)
         {
-            CPLFree(fileNameToWrite);
             continue;
         }
 
@@ -279,7 +293,6 @@ int main(int argc, char *argv[])
                 if (skip_different_projection)
                 {
                     CPLFree(fileNameToWrite);
-                    GDALClose( hDS );
                     continue;
                 }
             }
