@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dted_api.c 12272 2007-09-28 21:38:30Z rouault $
+ * $Id: dted_api.c 14979 2008-07-19 18:11:24Z rouault $
  *
  * Project:  DTED Translator
  * Purpose:  Implementation of DTED/CDED access functions.
@@ -29,7 +29,12 @@
 
 #include "dted_api.h"
 
-CPL_CVSID("$Id: dted_api.c 12272 2007-09-28 21:38:30Z rouault $");
+#ifndef AVOID_CPL
+CPL_CVSID("$Id: dted_api.c 14979 2008-07-19 18:11:24Z rouault $");
+#endif
+
+static int bWarnedTwoComplement = FALSE;
+
 
 /************************************************************************/
 /*                            DTEDGetField()                            */
@@ -40,12 +45,10 @@ CPL_CVSID("$Id: dted_api.c 12272 2007-09-28 21:38:30Z rouault $");
 /************************************************************************/
 
 static
-const char *DTEDGetField( const char *pachRecord, int nStart, int nSize )
+char *DTEDGetField( char szResult[81], const char *pachRecord, int nStart, int nSize )
 
 {
-    static char szResult[81];
-
-    CPLAssert( nSize < sizeof(szResult) );
+    CPLAssert( nSize < 81 );
     memcpy( szResult, pachRecord + nStart - 1, nSize );
     szResult[nSize] = '\0';
 
@@ -90,6 +93,8 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
     int deg = 0;
     int min = 0;
     int sec = 0;
+    int bSwapLatLong = FALSE;
+    char szResult[81];
 
 /* -------------------------------------------------------------------- */
 /*      Open the physical file.                                         */
@@ -103,14 +108,16 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
 
     if( fp == NULL )
     {
-#ifndef AVOID_CPL
         if( !bTestOpen )
         {
-            CPLError( CE_Failure, CPLE_AppDefined,
+#ifndef AVOID_CPL
+            CPLError( CE_Failure, CPLE_OpenFailed,
+#else
+            fprintf( stderr, 
+#endif
                       "Failed to open file %s.",
                       pszFilename );
         }
-#endif
 
         return NULL;
     }
@@ -123,12 +130,16 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
     {
         if( VSIFReadL( achRecord, 1, DTED_UHL_SIZE, fp ) != DTED_UHL_SIZE )
         {
-#ifndef AVOID_CPL
             if( !bTestOpen )
-                CPLError( CE_Failure, CPLE_OpenFailed,
+            {
+#ifndef AVOID_CPL
+               CPLError( CE_Failure, CPLE_OpenFailed,
+#else
+               fprintf( stderr, 
+#endif
                           "Unable to read header, %s is not DTED.",
                           pszFilename );
-#endif
+            }
             VSIFCloseL( fp );
             return NULL;
         }
@@ -137,13 +148,16 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
 
     if( !EQUALN(achRecord,"UHL",3) )
     {
-#ifndef AVOID_CPL
         if( !bTestOpen )
+        {
+#ifndef AVOID_CPL
             CPLError( CE_Failure, CPLE_OpenFailed,
+#else
+            fprintf( stderr, 
+#endif
                       "No UHL record.  %s is not a DTED file.",
                       pszFilename );
-#endif
-        
+        }
         VSIFCloseL( fp );
         return NULL;
     }
@@ -157,8 +171,8 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
 
     psDInfo->bUpdate = EQUAL(pszAccess,"r+b");
     
-    psDInfo->nXSize = atoi(DTEDGetField(achRecord,48,4));
-    psDInfo->nYSize = atoi(DTEDGetField(achRecord,52,4));
+    psDInfo->nXSize = atoi(DTEDGetField(szResult,achRecord,48,4));
+    psDInfo->nYSize = atoi(DTEDGetField(szResult,achRecord,52,4));
 
     psDInfo->nUHLOffset = VSIFTellL( fp ) - DTED_UHL_SIZE;
     psDInfo->pachUHLRecord = (char *) CPLMalloc(DTED_UHL_SIZE);
@@ -195,28 +209,49 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
 /*      center of the area.                                             */
 /* -------------------------------------------------------------------- */
     psDInfo->dfPixelSizeX =
-        atoi(DTEDGetField(achRecord,21,4)) / 36000.0;
+        atoi(DTEDGetField(szResult,achRecord,21,4)) / 36000.0;
 
     psDInfo->dfPixelSizeY =
-        atoi(DTEDGetField(achRecord,25,4)) / 36000.0;
+        atoi(DTEDGetField(szResult,achRecord,25,4)) / 36000.0;
 
     /* create a scope so I don't need to declare these up top */
-    deg = atoi(stripLeadingZeros(DTEDGetField(achRecord,5,3)));
-    min = atoi(stripLeadingZeros(DTEDGetField(achRecord,8,2)));
-    sec = atoi(stripLeadingZeros(DTEDGetField(achRecord,10,2)));
+    deg = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,5,3)));
+    min = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,8,2)));
+    sec = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,10,2)));
+
+    /* NOTE : The first version of MIL-D-89020 was buggy.
+       The latitude and longitude of the LL cornder of the UHF record was inverted.
+       This was fixed in MIL-D-89020 Amendement 1, but some products may be affected.
+       We detect this situation by looking at N/S in the longitude field and
+       E/W in the latitude one
+    */
 
     dfLLOriginX = deg + min / 60.0 + sec / 3600.0;
     if( achRecord[11] == 'W' )
         dfLLOriginX *= -1;
+    else if ( achRecord[11] == 'N' )
+        bSwapLatLong = TRUE;
+    else if ( achRecord[11] == 'S' )
+    {
+        dfLLOriginX *= -1;
+        bSwapLatLong = TRUE;
+    }
 
-    deg = atoi(stripLeadingZeros(DTEDGetField(achRecord,13,3)));
-    min = atoi(stripLeadingZeros(DTEDGetField(achRecord,16,2)));
-    sec = atoi(stripLeadingZeros(DTEDGetField(achRecord,18,2)));
+    deg = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,13,3)));
+    min = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,16,2)));
+    sec = atoi(stripLeadingZeros(DTEDGetField(szResult,achRecord,18,2)));
 
     dfLLOriginY = deg + min / 60.0 + sec / 3600.0;
-    if( achRecord[19] == 'S' )
+    if( achRecord[19] == 'S' || (bSwapLatLong && achRecord[19] == 'W'))
         dfLLOriginY *= -1;
 
+    if (bSwapLatLong)
+    {
+        double dfTmp = dfLLOriginX;
+        dfLLOriginX = dfLLOriginY;
+        dfLLOriginY = dfTmp;
+    }
+
     psDInfo->dfULCornerX = dfLLOriginX - 0.5 * psDInfo->dfPixelSizeX;
     psDInfo->dfULCornerY = dfLLOriginY - 0.5 * psDInfo->dfPixelSizeY
         + psDInfo->nYSize * psDInfo->dfPixelSizeY;
@@ -224,6 +259,80 @@ DTEDInfo * DTEDOpen( const char * pszFilename,
     return psDInfo;
 }
 
+
+
+/************************************************************************/
+/*                            DTEDReadPoint()                           */
+/*                                                                      */
+/*      Read one single sample. The coordinates are given from the      */
+/*      top-left corner of the file (contrary to the internal           */
+/*      organisation or a DTED file)                                    */
+/************************************************************************/
+
+int DTEDReadPoint( DTEDInfo * psDInfo, int nXOff, int nYOff, GInt16* panVal)
+{
+    int nOffset;
+    GByte pabyData[2];
+
+    if (nYOff < 0 || nXOff < 0 || nYOff >= psDInfo->nYSize || nXOff >= psDInfo->nXSize)
+    {
+#ifndef AVOID_CPL
+        CPLError( CE_Failure, CPLE_AppDefined,
+#else
+        fprintf( stderr, 
+#endif
+                  "Invalid raster coordinates (%d,%d) in DTED file.\n", nXOff, nYOff);
+        return FALSE;
+    }
+
+    nOffset = psDInfo->nDataOffset + nXOff * (12+psDInfo->nYSize*2) + 8 + 2 * (psDInfo->nYSize-1-nYOff);
+    if( VSIFSeekL( psDInfo->fp, nOffset, SEEK_SET ) != 0
+        || VSIFReadL( pabyData, 2, 1, psDInfo->fp ) != 1)
+    {
+#ifndef AVOID_CPL
+        CPLError( CE_Failure, CPLE_FileIO,
+#else
+        fprintf( stderr, 
+#endif
+                  "Failed to seek to, or read (%d,%d) at offset %d\n"
+                  "in DTED file.\n",
+                  nXOff, nYOff, nOffset );
+        return FALSE;
+    }
+
+    *panVal = ((pabyData[0] & 0x7f) << 8) | pabyData[1];
+
+    if( pabyData[0] & 0x80 )
+    {
+        *panVal *= -1;
+
+        /*
+        ** It seems that some files are improperly generated in twos
+        ** complement form for negatives.  For these, redo the job
+        ** in twos complement.  eg. w_069_s50.dt0
+        */
+        if(( *panVal < -16000 ) && (*panVal != DTED_NODATA_VALUE))
+        {
+            *panVal = (pabyData[0] << 8) | pabyData[1];
+
+            if( !bWarnedTwoComplement )
+            {
+                bWarnedTwoComplement = TRUE;
+#ifndef AVOID_CPL
+                CPLError( CE_Warning, CPLE_AppDefined,
+#else
+                fprintf( stderr,
+#endif
+                            "The DTED driver found values less than -16000, and has adjusted\n"
+                            "them assuming they are improperly two-complemented.  No more warnings\n"
+                            "will be issued in this session about this operation." );
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 /************************************************************************/
 /*                          DTEDReadProfile()                           */
 /*                                                                      */
@@ -272,7 +381,7 @@ int DTEDReadProfileEx( DTEDInfo * psDInfo, int nColumnOffset,
 /* -------------------------------------------------------------------- */
     for( i = 0; i < psDInfo->nYSize; i++ )
     {
-        panData[i] = (pabyRecord[8+i*2] & 0x7f) * 256 + pabyRecord[8+i*2+1];
+        panData[i] = ((pabyRecord[8+i*2] & 0x7f) << 8) | pabyRecord[8+i*2+1];
 
         if( pabyRecord[8+i*2] & 0x80 )
         {
@@ -285,15 +394,16 @@ int DTEDReadProfileEx( DTEDInfo * psDInfo, int nColumnOffset,
             */
             if(( panData[i] < -16000 ) && (panData[i] != DTED_NODATA_VALUE))
             {
-                static int bWarned = FALSE;
+                panData[i] = (pabyRecord[8+i*2] << 8) | pabyRecord[8+i*2+1];
 
-                memcpy( panData + i, pabyRecord + 8+i*2, 2 );
-                panData[i] = CPL_MSBWORD16( panData[i] );
-
-                if( !bWarned )
+                if( !bWarnedTwoComplement )
                 {
-                    bWarned = TRUE;
+                    bWarnedTwoComplement = TRUE;
+#ifndef AVOID_CPL
                     CPLError( CE_Warning, CPLE_AppDefined,
+#else
+                    fprintf( stderr,
+#endif
                               "The DTED driver found values less than -16000, and has adjusted\n"
                               "them assuming they are improperly two-complemented.  No more warnings\n"
                               "will be issued in this session about this operation." );
@@ -325,7 +435,11 @@ int DTEDReadProfileEx( DTEDInfo * psDInfo, int nColumnOffset,
             if (! bWarned)
             {
                 bWarned = TRUE;
+#ifndef AVOID_CPL
                 CPLError( CE_Warning, CPLE_AppDefined,
+#else
+                fprintf( stderr,
+#endif
                             "The DTED driver has read from the file a checksum "
                             "with an impossible value (0x%X) at column %d.\n"
                             "Check with your file producer.\n"
@@ -335,7 +449,11 @@ int DTEDReadProfileEx( DTEDInfo * psDInfo, int nColumnOffset,
         }
         else if (fileCheckSum != nCheckSum)
         {
-            CPLError( CE_Failure, CPLE_AppDefined,
+#ifndef AVOID_CPL
+            CPLError( CE_Warning, CPLE_AppDefined,
+#else
+            fprintf( stderr,
+#endif
                       "The DTED driver has found a computed and read checksum "
                       "that do not match at column %d.\n",
                       nColumnOffset);
