@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalallregister.cpp 13124 2007-11-28 00:08:49Z pvachon $
+ * $Id: gdalallregister.cpp 15659 2008-10-31 04:19:53Z ilucena $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALAllRegister(), primary format registration.
@@ -30,7 +30,7 @@
 #include "gdal_priv.h"
 #include "gdal_frmts.h"
 
-CPL_CVSID("$Id: gdalallregister.cpp 13124 2007-11-28 00:08:49Z pvachon $");
+CPL_CVSID("$Id: gdalallregister.cpp 15659 2008-10-31 04:19:53Z ilucena $");
 
 #ifdef notdef
 // we may have a use for this some day
@@ -226,27 +226,6 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_HDF4Image();
 #endif
 
-#ifdef FRMT_raw
-    GDALRegister_PNM();
-    GDALRegister_DOQ1();
-    GDALRegister_DOQ2();
-    GDALRegister_ENVI();
-    GDALRegister_EHdr();
-    GDALRegister_GenBin();
-    GDALRegister_PAux();
-    GDALRegister_MFF();
-    GDALRegister_HKV();
-    GDALRegister_FujiBAS();
-    GDALRegister_GSC();
-    GDALRegister_FAST();
-    GDALRegister_BT();
-    GDALRegister_LAN();
-    GDALRegister_CPG();
-    GDALRegister_IDA();
-    GDALRegister_NDF();
-    GDALRegister_DIPEx();
-#endif
-
 #ifdef FRMT_pds
     GDALRegister_ISIS3();
     GDALRegister_ISIS2();
@@ -306,11 +285,14 @@ void CPL_STDCALL GDALAllRegister()
     GDALRegister_SDE();
 #endif
 
-
 #ifdef FRMT_msgn
     GDALRegister_MSGN();
 #endif
 
+#ifdef FRMT_msg
+    GDALRegister_MSG();
+#endif
+
 #ifdef FRMT_idrisi
     GDALRegister_IDRISI();
 #endif
@@ -330,18 +312,52 @@ void CPL_STDCALL GDALAllRegister()
 #endif
 
 #ifdef FRMT_tsx
-	GDALRegister_TSX();
+    GDALRegister_TSX();
 #endif
 
 #ifdef FRMT_coasp
     GDALRegister_COASP();
 #endif
 
+#ifdef FRMT_tms
+    GDALRegister_TMS();
+#endif
+
+/* -------------------------------------------------------------------- */
+/*      Put raw formats at the end of the list. These drivers support   */
+/*      various ASCII-header labeled formats, so the driver could be    */
+/*      confused if you have files in some of above formats and such    */
+/*      ASCII-header in the same directory.                             */
+/* -------------------------------------------------------------------- */
+
+#ifdef FRMT_raw
+    GDALRegister_PNM();
+    GDALRegister_DOQ1();
+    GDALRegister_DOQ2();
+    GDALRegister_ENVI();
+    GDALRegister_EHdr();
+    GDALRegister_GenBin();
+    GDALRegister_PAux();
+    GDALRegister_MFF();
+    GDALRegister_HKV();
+    GDALRegister_FujiBAS();
+    GDALRegister_GSC();
+    GDALRegister_FAST();
+    GDALRegister_BT();
+    GDALRegister_LAN();
+    GDALRegister_CPG();
+    GDALRegister_IDA();
+    GDALRegister_NDF();
+    GDALRegister_EIR();
+    GDALRegister_DIPEx();
+    GDALRegister_LCP();
+#endif
 
 /* -------------------------------------------------------------------- */
 /*      Our test for the following is weak or expensive so we try       */
 /*      them last.                                                      */
 /* -------------------------------------------------------------------- */
+
 #ifdef FRMT_rik
     GDALRegister_RIK();
 #endif
@@ -374,6 +390,19 @@ void CPL_STDCALL GDALAllRegister()
 #ifdef FRMT_adrg
     GDALRegister_ADRG();
 #endif
+
+#ifdef FRMT_blx
+    GDALRegister_BLX();
+#endif
+
+#ifdef FRMT_pgchip
+    GDALRegister_PGCHIP();
+#endif
+
+#ifdef FRMT_georaster
+    GDALRegister_GEOR();
+#endif
+
 /* -------------------------------------------------------------------- */
 /*      Deregister any drivers explicitly marked as supressed by the    */
 /*      GDAL_SKIP environment variable.                                 */
