@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfaband.cpp 15164 2008-08-19 20:17:27Z rouault $
+ * $Id: hfaband.cpp 15711 2008-11-12 02:19:41Z warmerdam $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFABand, for accessing one Eimg_Layer.
@@ -32,7 +32,7 @@
 
 /* include the compression code */
 
-CPL_CVSID("$Id: hfaband.cpp 15164 2008-08-19 20:17:27Z rouault $");
+CPL_CVSID("$Id: hfaband.cpp 15711 2008-11-12 02:19:41Z warmerdam $");
 
 /************************************************************************/
 /*                              HFABand()                               */
@@ -57,6 +57,7 @@ HFABand::HFABand( HFAInfo_t * psInfoIn, HFAEntry * poNodeIn )
 
     nPCTColors = -1;
     apadfPCT[0] = apadfPCT[1] = apadfPCT[2] = apadfPCT[3] = NULL;
+    padfPCTBins = NULL;
 
     nOverviews = 0;
     papoOverviews = NULL;
@@ -66,13 +67,22 @@ HFABand::HFABand( HFAInfo_t * psInfoIn, HFAEntry * poNodeIn )
     bNoDataSet = FALSE;
     dfNoData = 0.0;
 
-    if (nBlockXSize <= 0 || nBlockYSize <= 0)
+    if (nWidth <= 0 || nHeight <= 0 || nBlockXSize <= 0 || nBlockYSize <= 0)
     {
+        nWidth = nHeight = 0;
         CPLError(CE_Failure, CPLE_AppDefined,
-                 "HFABand::HFABand : (nBlockXSize <= 0 || nBlockYSize <= 0)");
+                 "HFABand::HFABand : (nWidth <= 0 || nHeight <= 0 || nBlockXSize <= 0 || nBlockYSize <= 0)");
+        return;
+    }
+    if (HFAGetDataTypeBits(nDataType) == 0)
+    {
+        nWidth = nHeight = 0;
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "HFABand::HFABand : nDataType=%d unhandled", nDataType);
         return;
     }
 
+    /* FIXME? : risk of overflow in additions and multiplication */
     nBlocksPerRow = (nWidth + nBlockXSize - 1) / nBlockXSize;
     nBlocksPerColumn = (nHeight + nBlockYSize - 1) / nBlockYSize;
     nBlocks = nBlocksPerRow * nBlocksPerColumn;
@@ -174,6 +184,13 @@ HFABand::HFABand( HFAInfo_t * psInfoIn, HFAEntry * poNodeIn )
             papoOverviews = (HFABand **) 
                 CPLRealloc(papoOverviews, sizeof(void*) * ++nOverviews );
             papoOverviews[nOverviews-1] = new HFABand( psHFA, poOvEntry );
+            if (papoOverviews[nOverviews-1]->nWidth == 0)
+            {
+                nWidth = nHeight = 0;
+                delete papoOverviews[nOverviews-1];
+                papoOverviews[nOverviews-1] = NULL;
+                return;
+            }
         }
     }
 
@@ -195,6 +212,13 @@ HFABand::HFABand( HFAInfo_t * psInfoIn, HFAEntry * poNodeIn )
                 papoOverviews = (HFABand **) 
                     CPLRealloc(papoOverviews, sizeof(void*) * ++nOverviews );
                 papoOverviews[nOverviews-1] = new HFABand( psInfo, poChild );
+                if (papoOverviews[nOverviews-1]->nWidth == 0)
+                {
+                    nWidth = nHeight = 0;
+                    delete papoOverviews[nOverviews-1];
+                    papoOverviews[nOverviews-1] = NULL;
+                    return;
+                }
             }
         }
     }
@@ -224,6 +248,7 @@ HFABand::~HFABand()
     CPLFree( apadfPCT[1] );
     CPLFree( apadfPCT[2] );
     CPLFree( apadfPCT[3] );
+    CPLFree( padfPCTBins );
 
     if( fpExternal != NULL )
         VSIFCloseL( fpExternal );
@@ -254,9 +279,9 @@ CPLErr	HFABand::LoadBlockInfo()
         return CE_Failure;
     }
 
-    panBlockStart = (vsi_l_offset *)VSIMalloc(sizeof(vsi_l_offset) * nBlocks);
-    panBlockSize = (int *) VSIMalloc(sizeof(int) * nBlocks);
-    panBlockFlag = (int *) VSIMalloc(sizeof(int) * nBlocks);
+    panBlockStart = (vsi_l_offset *)VSIMalloc2(sizeof(vsi_l_offset), nBlocks);
+    panBlockSize = (int *) VSIMalloc2(sizeof(int), nBlocks);
+    panBlockFlag = (int *) VSIMalloc2(sizeof(int), nBlocks);
 
     if (panBlockStart == NULL || panBlockSize == NULL || panBlockFlag == NULL)
     {
@@ -360,7 +385,7 @@ CPLErr	HFABand::LoadExternalBlockInfo()
 /* -------------------------------------------------------------------- */
 /*      Allocate blockmap.                                              */
 /* -------------------------------------------------------------------- */
-    panBlockFlag = (int *) VSIMalloc(sizeof(int) * nBlocks);
+    panBlockFlag = (int *) VSIMalloc2(sizeof(int), nBlocks);
     if (panBlockFlag == NULL)
     {
         CPLError( CE_Failure, CPLE_OutOfMemory,
@@ -460,6 +485,9 @@ static CPLErr UncompressBlock( GByte *pabyCData, int /* nSrcBytes */,
         pabyValues = pabyCData + 13;
         nValueBitOffset = 0;
 
+/* -------------------------------------------------------------------- */
+/*      Loop over block pixels.                                         */
+/* -------------------------------------------------------------------- */
         for( nPixelsOutput = 0; nPixelsOutput < nMaxPixels; nPixelsOutput++ )
         {
             int	nDataValue, nRawValue;
@@ -1128,6 +1156,13 @@ CPLErr HFABand::SetRasterBlock( int nXBlock, int nYBlock, void * pData )
     int		iBlock;
     FILE	*fpData;
 
+    if( psInfo->eAccess == HFA_ReadOnly )
+    {
+        CPLError( CE_Failure, CPLE_NoWriteAccess,
+                  "Attempt to write block to read-only HFA file failed." );
+        return CE_Failure;
+    }
+
     if( LoadBlockInfo() != CE_None )
         return CE_Failure;
 
@@ -1462,6 +1497,63 @@ CPLErr HFABand::SetNoDataValue( double dfValue )
     return eErr;     
 }
 
+/************************************************************************/
+/*                           HFAReadPCTBins()                           */
+/*                                                                      */
+/*      Attempt to read the bins used for a PCT from a BinFunction      */
+/*      node.  On failure just return NULL.                             */
+/************************************************************************/
+
+static double *HFAReadPCTBins( HFAEntry *poBinFunc, int nPCTColors )
+
+{
+/* -------------------------------------------------------------------- */
+/*      First confirm this is a "BFUnique" bin function.  We don't      */
+/*      know what to do with any other types.                           */
+/* -------------------------------------------------------------------- */
+    const char *pszBinFunctionType = 
+        poBinFunc->GetStringField( "binFunction.type.string" );
+
+    if( pszBinFunctionType == NULL 
+        || !EQUAL(pszBinFunctionType,"BFUnique") )
+        return NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Process dictionary.                                             */
+/* -------------------------------------------------------------------- */
+    HFADictionary oMiniDict( 
+        poBinFunc->GetStringField( "binFunction.MIFDictionary" ) );
+
+    HFAType *poBFUnique = oMiniDict.FindType( "BFUnique" );
+    if( poBFUnique == NULL )
+        return NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Field the MIFObject raw data pointer.                           */
+/* -------------------------------------------------------------------- */
+    const GByte *pabyMIFObject = (const GByte *) 
+        poBinFunc->GetStringField("binFunction.MIFObject");
+    
+    if( pabyMIFObject == NULL )
+        return NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Decode bins.                                                    */
+/* -------------------------------------------------------------------- */
+    double *padfBins = (double *) CPLCalloc(sizeof(double),nPCTColors);
+    int i;
+
+    memcpy( padfBins, pabyMIFObject + 24, sizeof(double) * nPCTColors );
+    
+    for( i = 0; i < nPCTColors; i++ )
+    {
+        HFAStandard( 8, padfBins + i );
+//        CPLDebug( "HFA", "Bin[%d] = %g", i, padfBins[i] );
+    }
+    
+    return padfBins;
+}
+
 /************************************************************************/
 /*                               GetPCT()                               */
 /*                                                                      */
@@ -1472,7 +1564,8 @@ CPLErr HFABand::GetPCT( int * pnColors,
                         double **ppadfRed,
                         double **ppadfGreen,
                         double **ppadfBlue,
-                        double **ppadfAlpha )
+                        double **ppadfAlpha,
+                        double **ppadfBins )
 
 {
     *pnColors = 0;
@@ -1480,6 +1573,7 @@ CPLErr HFABand::GetPCT( int * pnColors,
     *ppadfGreen = NULL;
     *ppadfBlue = NULL;
     *ppadfAlpha = NULL;
+    *ppadfBins = NULL;
         
 /* -------------------------------------------------------------------- */
 /*      If we haven't already tried to load the colors, do so now.      */
@@ -1495,10 +1589,17 @@ CPLErr HFABand::GetPCT( int * pnColors,
         if( poColumnEntry == NULL )
             return( CE_Failure );
 
+        /* FIXME? : we could also check that nPCTColors is not too big */
         nPCTColors = poColumnEntry->GetIntField( "numRows" );
         for( iColumn = 0; iColumn < 4; iColumn++ )
         {
-            apadfPCT[iColumn] = (double *)CPLMalloc(sizeof(double)*nPCTColors);
+            apadfPCT[iColumn] = (double *)VSIMalloc2(sizeof(double),nPCTColors);
+            if (apadfPCT[iColumn] == NULL)
+            {
+                CPLError(CE_Failure, CPLE_OutOfMemory, "Color palette will be ignored");
+                return CE_Failure;
+            }
+
             if( iColumn == 0 )
                 poColumnEntry = poNode->GetNamedChild("Descriptor_Table.Red");
             else if( iColumn == 1 )
@@ -1517,15 +1618,36 @@ CPLErr HFABand::GetPCT( int * pnColors,
             }
             else
             {
-                VSIFSeekL( psInfo->fp, poColumnEntry->GetIntField("columnDataPtr"),
-                           SEEK_SET );
-                VSIFReadL( apadfPCT[iColumn], sizeof(double), nPCTColors,
-                           psInfo->fp);
+                if (VSIFSeekL( psInfo->fp, poColumnEntry->GetIntField("columnDataPtr"),
+                               SEEK_SET ) < 0)
+                {
+                    CPLError( CE_Failure, CPLE_FileIO,
+                              "VSIFSeekL() failed in HFABand::GetPCT()." );
+                    return CE_Failure;
+                }
+                if (VSIFReadL( apadfPCT[iColumn], sizeof(double), nPCTColors,
+                               psInfo->fp) != (size_t)nPCTColors)
+                {
+                    CPLError( CE_Failure, CPLE_FileIO,
+                              "VSIFReadL() failed in HFABand::GetPCT()." );
+                    return CE_Failure;
+                }
                 
                 for( i = 0; i < nPCTColors; i++ )
                     HFAStandard( 8, apadfPCT[iColumn] + i );
             }
         }
+
+/* -------------------------------------------------------------------- */
+/*      Do we have a custom binning function? If so, try reading it.    */
+/* -------------------------------------------------------------------- */
+        HFAEntry *poBinFunc = 
+            poNode->GetNamedChild("Descriptor_Table.#Bin_Function840#");
+        
+        if( poBinFunc != NULL )
+        {
+            padfPCTBins = HFAReadPCTBins( poBinFunc, nPCTColors );
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -1539,6 +1661,7 @@ CPLErr HFABand::GetPCT( int * pnColors,
     *ppadfGreen = apadfPCT[1];
     *ppadfBlue = apadfPCT[2];
     *ppadfAlpha = apadfPCT[3];
+    *ppadfBins = padfPCTBins;
     
     return( CE_None );
 }
