@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfaentry.cpp 12415 2007-10-14 03:45:09Z warmerdam $
+ * $Id: hfaentry.cpp 15768 2008-11-20 05:17:40Z warmerdam $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFAEntry class for reading and relating
@@ -37,7 +37,7 @@
 #include "hfa_p.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: hfaentry.cpp 12415 2007-10-14 03:45:09Z warmerdam $");
+CPL_CVSID("$Id: hfaentry.cpp 15768 2008-11-20 05:17:40Z warmerdam $");
 
 /************************************************************************/
 /*                              HFAEntry()                              */
@@ -53,6 +53,7 @@ HFAEntry::HFAEntry( HFAInfo_t * psHFAIn, GUInt32 nPos,
     
     nFilePos = nPos;
     bDirty = FALSE;
+    bIsMIFObject = FALSE;
 
     poParent = poParentIn;
     poPrev = poPrevIn;
@@ -126,6 +127,7 @@ HFAEntry::HFAEntry( HFAInfo_t * psHFAIn,
     psHFA = psHFAIn;
     
     nFilePos = 0;
+    bIsMIFObject = FALSE;
 
     poParent = poParentIn;
     poPrev = poNext = poChild = NULL;
@@ -165,6 +167,75 @@ HFAEntry::HFAEntry( HFAInfo_t * psHFAIn,
     MarkDirty();
 }
 
+/************************************************************************/
+/*                              HFAEntry()                              */
+/*                                                                      */
+/*      Create a pseudo-HFAEntry wrapping a MIFObject.                  */
+/************************************************************************/
+
+HFAEntry::HFAEntry( HFAEntry * poContainer,
+                    const char *pszMIFObjectPath )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Initialize Entry                                                */
+/* -------------------------------------------------------------------- */
+    nFilePos = 0;
+
+    poParent = poPrev = poNext = poChild = NULL;
+
+    bIsMIFObject = TRUE;
+
+    nDataPos = nDataSize = 0;
+    nNextPos = nChildPos = 0;
+
+    memset( szName, 0, 64 );
+
+    pabyData = NULL;
+    poType = NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Create a dummy HFAInfo_t.                                       */
+/* -------------------------------------------------------------------- */
+    psHFA = (HFAInfo_t *) CPLCalloc(sizeof(HFAInfo_t),1);
+
+    psHFA->eAccess = HFA_ReadOnly;
+    psHFA->bTreeDirty = FALSE;
+    psHFA->poRoot = this;
+
+    psHFA->poDictionary = new HFADictionary( 
+        poContainer->GetStringField( 
+            CPLString().Printf("%s.%s", pszMIFObjectPath, "MIFDictionary" ) ) );
+
+/* -------------------------------------------------------------------- */
+/*      Work out the type for this MIFObject.                           */
+/* -------------------------------------------------------------------- */
+    memset( szType, 0, 32 );
+    strncpy( szType, 
+             poContainer->GetStringField( 
+                 CPLString().Printf("%s.%s", pszMIFObjectPath, "type.string") ),
+             32 );
+    
+    poType = psHFA->poDictionary->FindType( szType );
+
+/* -------------------------------------------------------------------- */
+/*      Find the desired field.                                         */
+/* -------------------------------------------------------------------- */
+    GInt32 nMIFObjectSize;
+    const GByte *pabyRawData = (const GByte *) 
+        poContainer->GetStringField( 
+            CPLString().Printf("%s.%s", pszMIFObjectPath, "MIFObject" ) );
+
+    // we rudely look before the field data to get at the pointer/size info
+    memcpy( &nMIFObjectSize, pabyRawData-8, 4 );
+    HFAStandard( 4, &nMIFObjectSize );
+
+    nDataSize = nMIFObjectSize;
+    pabyData = (GByte *) VSIMalloc(nDataSize);
+
+    memcpy( pabyData, pabyRawData, nDataSize );
+}
+
 /************************************************************************/
 /*                             ~HFAEntry()                              */
 /*                                                                      */
@@ -182,6 +253,12 @@ HFAEntry::~HFAEntry()
 
     if( poChild != NULL )
         delete poChild;
+
+    if( bIsMIFObject )
+    {
+        delete psHFA->poDictionary;
+        CPLFree( psHFA );
+    }
 }
 
 /************************************************************************/
@@ -267,7 +344,14 @@ void HFAEntry::LoadData()
 /* -------------------------------------------------------------------- */
 /*      Allocate buffer, and read data.                                 */
 /* -------------------------------------------------------------------- */
-    pabyData = (GByte *) CPLMalloc(nDataSize);
+    pabyData = (GByte *) VSIMalloc(nDataSize);
+    if (pabyData == NULL)
+    {
+        CPLError( CE_Failure, CPLE_OutOfMemory,
+                  "VSIMalloc() failed in HFAEntry::LoadData()." );
+        return;
+    }
+    
     if( VSIFSeekL( psHFA->fp, nDataPos, SEEK_SET ) < 0 )
     {
         CPLError( CE_Failure, CPLE_FileIO,
@@ -353,7 +437,7 @@ void HFAEntry::DumpFieldValues( FILE * fp, const char * pszPrefix )
 
     LoadData();
 
-    if( poType == NULL )
+    if( pabyData == NULL || poType == NULL )
         return;
 
     poType->DumpInstValue( fp,
@@ -740,7 +824,7 @@ CPLErr HFAEntry::FlushToDisk()
 /* -------------------------------------------------------------------- */
         GUInt32		nLong;
 
-        VSIFFlushL( psHFA->fp );
+        //VSIFFlushL( psHFA->fp );
         if( VSIFSeekL( psHFA->fp, nFilePos, SEEK_SET ) != 0 )
         {
             CPLError( CE_Failure, CPLE_FileIO, 
@@ -795,7 +879,7 @@ CPLErr HFAEntry::FlushToDisk()
 /* -------------------------------------------------------------------- */
 /*      Write out the data.                                             */
 /* -------------------------------------------------------------------- */
-        VSIFFlushL( psHFA->fp );
+        //VSIFFlushL( psHFA->fp );
         if( nDataSize > 0 && pabyData != NULL )
         {
             if( VSIFSeekL( psHFA->fp, nDataPos, SEEK_SET ) != 0 
@@ -809,7 +893,7 @@ CPLErr HFAEntry::FlushToDisk()
             }
         }
 
-        VSIFFlushL( psHFA->fp );
+        //VSIFFlushL( psHFA->fp );
     }
 
 /* -------------------------------------------------------------------- */
@@ -842,3 +926,4 @@ void HFAEntry::MarkDirty()
     bDirty = TRUE;
     psHFA->bTreeDirty = TRUE;
 }
+
