@@ -1,5 +1,5 @@
 /*****************************************************************************
- * $Id: IntergraphBand.cpp 12578 2007-10-28 22:44:53Z ilucena $
+ * $Id: IntergraphBand.cpp 15581 2008-10-22 15:09:37Z warmerdam $
  *
  * Project:  Intergraph Raster Format support
  * Purpose:  Read/Write Intergraph Raster Format, band support
@@ -65,8 +65,13 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDS,
     // Get Header Info
     // -------------------------------------------------------------------- 
 
-    memcpy( &hHeaderOne, &poDS->hHeaderOne, SIZEOF_HDR1 );
-    memcpy( &hHeaderTwo, &poDS->hHeaderTwo, SIZEOF_HDR2_A );
+    GByte abyBuf[MAX(SIZEOF_HDR1,SIZEOF_HDR2_A)];
+
+    INGR_HeaderOneMemToDisk( &poDS->hHeaderOne, abyBuf );
+    INGR_HeaderOneDiskToMem( &hHeaderOne, abyBuf );
+
+    INGR_HeaderTwoAMemToDisk( &poDS->hHeaderTwo, abyBuf );
+    INGR_HeaderTwoADiskToMem( &hHeaderTwo, abyBuf );
 
     // -------------------------------------------------------------------- 
     // Get the image start from Words to Follow (WTF)
@@ -144,7 +149,7 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDS,
     // Get the Data Type from Format
     // -------------------------------------------------------------------- 
 
-    this->eDataType = INGR_GetDataType( eFormat );
+    this->eDataType = INGR_GetDataType( (uint16) eFormat );
 
     // -------------------------------------------------------------------- 
     // Allocate buffer for a Block of data
@@ -159,7 +164,7 @@ IntergraphRasterBand::IntergraphRasterBand( IntergraphDataset *poDS,
     // More Metadata Information
     // -------------------------------------------------------------------- 
 
-    SetMetadataItem( "FORMAT", INGR_GetFormatName( eFormat ), 
+    SetMetadataItem( "FORMAT", INGR_GetFormatName( (uint16) eFormat ), 
         "IMAGE_STRUCTURE" );
 
     if( bTiled )
@@ -335,7 +340,7 @@ CPLErr IntergraphRasterBand::IReadBlock( int nBlockXOff,
     {
         memset( pImage, 0, nBlockBufSize );
         CPLError( CE_Failure, CPLE_FileIO, 
-            "Can't read (%s) tile with X offset %d and Y offset %d.\n%s", 
+            "Can't read (%s) tile with X offset %d and Y offset %d.\n", 
             ((IntergraphDataset*)poDS)->pszFilename, nBlockXOff, nBlockYOff );
         return CE_Failure;
     }
@@ -379,7 +384,7 @@ IntergraphRGBBand::IntergraphRGBBand( IntergraphDataset *poDS,
                                      int nRGorB )
     : IntergraphRasterBand( poDS, nBand, nBandOffset )
 {
-    nRGBIndex     = nRGorB;
+    nRGBIndex     = (uint8) nRGorB;
 
     // -------------------------------------------------------------------- 
     // Reallocate buffer for a block of RGB Data
@@ -431,8 +436,10 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDS,
                                      int nRGorB )
     : IntergraphRasterBand( poDS, nBand, nBandOffset )
 {
-    nRLESize    = 0;
-    nRGBIndex   = nRGorB;
+    nRLESize         = 0;
+    nRGBIndex        = (uint8) nRGorB;
+    bRLEBlockLoaded  = FALSE;
+    panRLELineOffset = NULL;
 
     if( ! this->bTiled )
     {
@@ -441,9 +448,20 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDS,
         // ------------------------------------------------------------
 
         nFullBlocksX = 1;
-        nFullBlocksY = 1;
 
-        nBlockYSize  = nRasterYSize;
+        if( eFormat == RunLengthEncodedC || eFormat == RunLengthEncoded )
+        {
+            nBlockYSize = 1;
+            panRLELineOffset = (uint32 *) 
+                CPLCalloc(sizeof(uint32),nRasterYSize);
+            nFullBlocksY = nRasterYSize;
+        }
+        else
+        {
+            nBlockYSize  = nRasterYSize;
+            nFullBlocksY = 1;
+        }
+
         nRLESize     = INGR_GetDataBlockSize( poDS->pszFilename, 
                           hHeaderTwo.CatenatedFilePointer,
                           nDataOffset);
@@ -486,10 +504,10 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDS,
     // Set a black and white Color Table
     // ----------------------------------------------------------------
 
-	if( eFormat == RunLengthEncoded )
-	{
-        BlackWhiteCT();
-	}
+    if( eFormat == RunLengthEncoded )
+    {
+        BlackWhiteCT( true );
+    }
 
 }
 
@@ -500,6 +518,7 @@ IntergraphRLEBand::IntergraphRLEBand( IntergraphDataset *poDS,
 IntergraphRLEBand::~IntergraphRLEBand()
 {
     CPLFree( pabyRLEBlock );
+    CPLFree( panRLELineOffset );
 }
 
 //  ----------------------------------------------------------------------------
@@ -514,7 +533,15 @@ CPLErr IntergraphRLEBand::IReadBlock( int nBlockXOff,
     // Load Block Buffer
     // --------------------------------------------------------------------
 
-    uint32 nBytesRead = LoadBlockBuf( nBlockXOff, nBlockYOff, nRLESize, pabyRLEBlock );
+    uint32 nBytesRead;
+    
+    if( bTiled || !bRLEBlockLoaded )
+    {
+        nBytesRead = LoadBlockBuf( nBlockXOff, nBlockYOff, nRLESize, pabyRLEBlock );
+        bRLEBlockLoaded = TRUE;
+    }
+    else
+        nBytesRead = nRLESize;
 
     if( nBytesRead == 0 )
     {
@@ -547,19 +574,44 @@ CPLErr IntergraphRLEBand::IReadBlock( int nBlockXOff,
     // Decode Run Length
     // --------------------------------------------------------------------
 
-    switch( eFormat )
+    if( bTiled && eFormat == RunLengthEncoded )
     {
-    case RunLengthEncoded:
-        nBytesRead = INGR_DecodeRunLengthBitonal( pabyRLEBlock, pabyBlockBuf,  
-            nRLESize, nBlockBufSize );
-        break;
-    case RunLengthEncodedC:
-        nBytesRead = INGR_DecodeRunLengthPaletted( pabyRLEBlock, pabyBlockBuf,  
-            nRLESize, nBlockBufSize );
-        break;
-    default:
-        nBytesRead = INGR_DecodeRunLength( pabyRLEBlock, pabyBlockBuf,  
-            nRLESize, nBlockBufSize );
+        nBytesRead = 
+            INGR_DecodeRunLengthBitonalTiled( pabyRLEBlock, pabyBlockBuf,  
+                                              nRLESize, nBlockBufSize, NULL );
+    }
+    
+    else if( bTiled || panRLELineOffset == NULL )
+    {
+        nBytesRead = INGR_Decode( eFormat, pabyRLEBlock, pabyBlockBuf,  
+                                  nRLESize, nBlockBufSize, 
+                                  NULL );
+    }
+
+    else
+    {
+        // If we are missing the offset to this line, process all
+        // preceding lines.
+        if( nBlockYOff > 0 && panRLELineOffset[nBlockYOff] == 0 )
+        {
+            int iLine;
+            for( iLine = 0; iLine < nBlockYOff; iLine++ )
+                IReadBlock( 0, iLine, pImage );
+        } 
+        if( nBlockYOff == 0 || panRLELineOffset[nBlockYOff] > 0 )
+        {
+            uint32 nBytesConsumed;
+            
+            nBytesRead = 
+                INGR_Decode( eFormat,
+                             pabyRLEBlock + panRLELineOffset[nBlockYOff], 
+                             pabyBlockBuf,  nRLESize, nBlockBufSize,
+                             &nBytesConsumed );
+            
+            if( nBlockYOff < nRasterYSize-1 )
+                panRLELineOffset[nBlockYOff+1] = 
+                    panRLELineOffset[nBlockYOff] + nBytesConsumed;
+        }
     }
 
     // --------------------------------------------------------------------
@@ -944,21 +996,28 @@ void IntergraphRasterBand::FlushBandHeader( void )
     }
 
     VSIFSeekL( poGDS->fp, nBandStart, SEEK_SET );
-#ifdef CPL_MSB
-    INGR_HeaderOne hdr1ForDisk;
-    memcpy(&hdr1ForDisk, &hHeaderOne, SIZEOF_HDR1);
-    INGR_HeaderOneMemToDisk(&hdr1ForDisk);
-    VSIFWriteL( &hdr1ForDisk, 1, SIZEOF_HDR1,   poGDS->fp );
 
-    INGR_HeaderTwoA hdr2ForDisk;
-    memcpy(&hdr2ForDisk, &hHeaderTwo, SIZEOF_HDR2_A);
-    INGR_HeaderTwoAMemToDisk(&hdr2ForDisk);
-    VSIFWriteL( &hdr2ForDisk, 1, SIZEOF_HDR2_A, poGDS->fp );
-#else
-    VSIFWriteL( &hHeaderOne, 1, SIZEOF_HDR1,   poGDS->fp );
-    VSIFWriteL( &hHeaderTwo, 1, SIZEOF_HDR2_A, poGDS->fp );
-#endif
-    VSIFWriteL( &hCTab,      1, SIZEOF_CTAB,   poGDS->fp );
+    GByte abyBuf[MAX(SIZEOF_HDR1,SIZEOF_CTAB)];
+
+    INGR_HeaderOneMemToDisk( &hHeaderOne, abyBuf );
+
+    VSIFWriteL( abyBuf, 1, SIZEOF_HDR1, poGDS->fp );
+
+    INGR_HeaderTwoAMemToDisk( &hHeaderTwo, abyBuf );
+
+    VSIFWriteL( abyBuf, 1, SIZEOF_HDR2_A, poGDS->fp );
+
+    unsigned int i = 0;
+    unsigned int n = 0;
+
+    for( i = 0; i < 256; i++ )
+    {
+        STRC2BUF( abyBuf, n, hCTab.Entry[i].v_red );
+        STRC2BUF( abyBuf, n, hCTab.Entry[i].v_green );
+        STRC2BUF( abyBuf, n, hCTab.Entry[i].v_blue );
+    }
+
+    VSIFWriteL( abyBuf, 1, SIZEOF_CTAB, poGDS->fp );
 }
 
 //  ----------------------------------------------------------------------------
