@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: msgndataset.cpp 11698 2007-06-25 16:33:06Z warmerdam $
+ * $Id: msgndataset.cpp 14563 2008-05-28 22:53:30Z rouault $
  *
  * Project:  MSG Native Reader
  * Purpose:  All code for EUMETSAT Archive format reader
@@ -33,7 +33,7 @@
 #include "msg_reader_core.h"
 using namespace msg_native_format;
 
-CPL_CVSID("$Id: msgndataset.cpp 11698 2007-06-25 16:33:06Z warmerdam $");
+CPL_CVSID("$Id: msgndataset.cpp 14563 2008-05-28 22:53:30Z rouault $");
 
 CPL_C_START
 void   GDALRegister_MSGN(void);
@@ -521,7 +521,7 @@ void GDALRegister_MSGN()
         poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
                                    "EUMETSAT Archive native (.nat)" );
         poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC,
-                                   "frmt_various.html#MSGN" );
+                                   "frmt_msgn.html" );
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "nat" );
 
         poDriver->pfnOpen = MSGNDataset::Open;
