@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitffile.c 13207 2007-12-03 05:19:11Z warmerdam $
+ * $Id: nitffile.c 15339 2008-09-08 07:41:21Z warmerdam $
  *
  * Project:  NITF Read/Write Library
  * Purpose:  Module responsible for opening NITF file, populating NITFFile
@@ -33,11 +33,16 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: nitffile.c 13207 2007-12-03 05:19:11Z warmerdam $");
+CPL_CVSID("$Id: nitffile.c 15339 2008-09-08 07:41:21Z warmerdam $");
 
-static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE, int nTRECount,
+static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE, 
                             int *pnOffset, int nBytesAvailable,
                             char **papszOptions );
+static int NITFWriteTREsFromOptions(
+    char *pachUDIDL, char *pachTRE,
+    int *pnOffset, int nBytesAvailable,
+    char **papszOptions );
+
 static int 
 NITFCollectSegmentInfo( NITFFile *psFile, int nOffset, char *pszType,
                         int nHeaderLenSize, int nDataLenSize, 
@@ -83,6 +88,7 @@ NITFFile *NITFOpen( const char *pszFilename, int bUpdatable )
         CPLError( CE_Failure, CPLE_AppDefined, 
                   "The file %s is not an NITF file.", 
                   pszFilename );
+        VSIFCloseL(fp);
         return NULL;
     }
 
@@ -95,6 +101,7 @@ NITFFile *NITFOpen( const char *pszFilename, int bUpdatable )
         CPLError( CE_Failure, CPLE_NotSupported, 
                   "Unable to read FSDWNG field from NITF file.  File is either corrupt\n"
                   "or empty." );
+        VSIFCloseL(fp);
         return NULL;
     }
 
@@ -112,6 +119,7 @@ NITFFile *NITFOpen( const char *pszFilename, int bUpdatable )
         CPLError( CE_Failure, CPLE_NotSupported, 
                   "Unable to read header length from NITF file.  File is either corrupt\n"
                   "or empty." );
+        VSIFCloseL(fp);
         return NULL;
     }
 
@@ -125,6 +133,7 @@ NITFFile *NITFOpen( const char *pszFilename, int bUpdatable )
         CPLError( CE_Failure, CPLE_NotSupported, 
                   "NITF Header Length (%d) seems to be corrupt.",
                   nHeaderLen );
+        VSIFCloseL(fp);
         return NULL;
     }
 
@@ -339,12 +348,15 @@ int NITFCreate( const char *pszFilename,
     FILE	*fp;
     char        *pachIMHDR;
     char        achHeader[5000];
+    int         nHeaderUsed = 0;
     int         nOffset = 0, iBand, nIHSize, nNPPBH, nNPPBV;
     GIntBig     nImageSize;
     int         nNBPR, nNBPC;
     const char *pszIREP;
     const char *pszIC = CSLFetchNameValue(papszOptions,"IC");
     const char *pszCLevel;
+    const char *pszOpt;
+    int nHL, nNUMT = 0;
     int nUDIDLOffset;
     const char *pszVersion;
 
@@ -371,6 +383,10 @@ int NITFCreate( const char *pszFilename,
     if( pszIREP == NULL )
         pszIREP = "MONO";
 
+    pszOpt = CSLFetchNameValue( papszOptions, "NUMT" );
+    if( pszOpt != NULL )
+        nNUMT = atoi(pszOpt);
+    
 /* -------------------------------------------------------------------- */
 /*      Compute raw image size, blocking factors and so forth.          */
 /* -------------------------------------------------------------------- */
@@ -473,22 +489,36 @@ int NITFCreate( const char *pszFilename,
     OVR(24,achHeader+300, ONAME        ,""                              );
     OVR(18,achHeader+324, OPHONE       ,""                              );
     PLACE (achHeader+342, FL           ,"????????????"                  );
-    PLACE (achHeader+354, HL           ,"000404"                        );
+    PLACE (achHeader+354, HL           ,"??????"                        );
     PLACE (achHeader+360, NUMI         ,"001"                           );
     PLACE (achHeader+363, LISH1        ,"??????"                        );
     PLACE (achHeader+369, LI1          ,CPLSPrintf("%010ud",(GUInt32) nImageSize)  );
     PLACE (achHeader+379, NUMS         ,"000"                           );
     PLACE (achHeader+382, NUMX         ,"000"                           );
-    PLACE (achHeader+385, NUMT         ,"000"                           );
-    PLACE (achHeader+388, NUMDES       ,"000"                           );
-    PLACE (achHeader+391, NUMRES       ,"000"                           );
-    PLACE (achHeader+394, UDHDL        ,"00000"                         );
-    PLACE (achHeader+399, XHDL         ,"00000"                         );
+    PLACE (achHeader+385, NUMT         ,CPLSPrintf("%03d",nNUMT)        );
+
+    PLACE (achHeader+388, LTSHnLTn     ,""                              );
+
+    nHL = 388 + (4+5) * nNUMT;
+
+    PLACE (achHeader+nHL, NUMDES       ,"000"                           );
+    nHL += 3;
+    PLACE (achHeader+nHL, NUMRES       ,"000"                           );
+    nHL += 3;
+    PLACE (achHeader+nHL, UDHDL        ,"00000"                         );
+    nHL += 5;
+    PLACE (achHeader+nHL, XHDL         ,"00000"                         );
+    nHL += 5;
+
+    // update header length
+    PLACE (achHeader+354, HL           ,CPLSPrintf("%06d",nHL)          );
     
+    nHeaderUsed = nHL;
+
 /* -------------------------------------------------------------------- */
 /*      Prepare the image header.                                       */
 /* -------------------------------------------------------------------- */
-    pachIMHDR = achHeader + 404;
+    pachIMHDR = achHeader + nHeaderUsed;
 
     PLACE (pachIMHDR+  0, IM           , "IM"                           );
     OVR(10,pachIMHDR+  2, IID1         , "Missing"                      );
@@ -627,26 +657,37 @@ int NITFCreate( const char *pszFilename,
     {
         NITFWriteBLOCKA( pachIMHDR + nUDIDLOffset, 
                          pachIMHDR + nOffset, 
-                         0, &nOffset, 
+                         &nOffset, 
                          sizeof(achHeader) - (pachIMHDR+nOffset-achHeader),
                          papszOptions );
     }
 
+    if( CSLFetchNameValue(papszOptions,"TRE") != NULL )
+    {
+        NITFWriteTREsFromOptions( 
+            pachIMHDR + nUDIDLOffset, 
+            pachIMHDR + nOffset, 
+            &nOffset, 
+            sizeof(achHeader) - (pachIMHDR+nOffset-achHeader),
+            papszOptions );
+    }
 
 /* -------------------------------------------------------------------- */
-/*      Update the image header length in the file header and the       */
-/*      total file size.                                                */
+/*      Update the image header length in the file header.              */
 /* -------------------------------------------------------------------- */
     nIHSize = nOffset;
 
     PLACE(achHeader+ 363, LISH1, CPLSPrintf("%06d",nIHSize)      );
-    PLACE(achHeader+ 342, FL,
-          CPLSPrintf( "%012d", 404 + nIHSize + nImageSize ) );
+
+    nHeaderUsed += nIHSize;
 
 /* -------------------------------------------------------------------- */
-/*      Write header info to file.                                      */
+/*      Update total file length, and write header info to file.        */
 /* -------------------------------------------------------------------- */
-    VSIFWriteL( achHeader, 1, nIHSize+404, fp );
+    PLACE(achHeader+ 342, FL,
+          CPLSPrintf( "%012d", (int) (nHeaderUsed + nImageSize) ) );
+
+    VSIFWriteL( achHeader, 1, nIHSize+nHL, fp );
 
 /* -------------------------------------------------------------------- */
 /*      Grow file to full required size by writing one byte at the end. */
@@ -669,11 +710,124 @@ int NITFCreate( const char *pszFilename,
     return TRUE;
 }
 
+/************************************************************************/
+/*                            NITFWriteTRE()                            */
+/************************************************************************/
+
+static int NITFWriteTRE( char *pachUDIDL, 
+                         char *pachTREInHeader, 
+                         int  *pnOffset, int nBytesAvailable,
+                         char *pszTREName, char *pabyTREData, int nTREDataSize )
+
+{
+    char szTemp[200];
+    int  nOldOffset;
+
+/* -------------------------------------------------------------------- */
+/*      Try to allocate space in the header for for this BLOCKA         */
+/*      TRE.                                                            */
+/* -------------------------------------------------------------------- */
+    if( nBytesAvailable < nTREDataSize+14 )
+    {
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "%s TRE not written due to lack of header space.",
+                  pszTREName );
+        return FALSE;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Update IXSHDL.                                                  */
+/* -------------------------------------------------------------------- */
+    nOldOffset = atoi(NITFGetField( szTemp, pachUDIDL, 5, 5 ));
+
+    if( nOldOffset == 0 )
+    {
+        nOldOffset = 3;
+        PLACE(pachUDIDL+10, IXSOFL, "000" );
+        *pnOffset += 3;
+    }
+    sprintf( szTemp, "%05d", nOldOffset + 11 + nTREDataSize );
+    PLACE( pachUDIDL + 5, IXSHDL, szTemp );
+
+/* -------------------------------------------------------------------- */
+/*      Create TRE prefix.                                              */
+/* -------------------------------------------------------------------- */
+    sprintf( pachTREInHeader + nOldOffset, "%-6s%05d", 
+             pszTREName, nTREDataSize );
+    memcpy( pachTREInHeader + nOldOffset + 11, 
+            pabyTREData, nTREDataSize );
+
+/* -------------------------------------------------------------------- */
+/*      Increment values.                                               */
+/* -------------------------------------------------------------------- */
+    *pnOffset += nTREDataSize + 11;
+
+    return TRUE;
+}
+
+/************************************************************************/
+/*                   NITFWriteTREsFromOptions()                         */
+/************************************************************************/
+
+static int NITFWriteTREsFromOptions(
+    char *pachUDIDL, char *pachTRE,
+    int *pnOffset, int nBytesAvailable,
+    char **papszOptions )    
+
+{
+    int bIgnoreBLOCKA = 
+        CSLFetchNameValue(papszOptions,"BLOCKA_BLOCK_COUNT") != NULL;
+    int iOption;
+
+    if( papszOptions == NULL )
+        return TRUE;
+
+    for( iOption = 0; papszOptions[iOption] != NULL; iOption++ )
+    {
+        const char *pszEscapedContents;
+        char *pszUnescapedContents;
+        char *pszTREName;
+        int  nContentLength;
+
+        if( !EQUALN(papszOptions[iOption],"TRE=",4) )
+            continue;
+
+        if( EQUALN(papszOptions[iOption]+4,"BLOCKA=",7)
+            && bIgnoreBLOCKA )
+            continue;
+
+        pszEscapedContents = CPLParseNameValue( papszOptions[iOption]+4, 
+                                                &pszTREName );
+        if (pszEscapedContents == NULL)
+        {
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "Could not parse creation options %s", papszOptions[iOption]+4);
+            return FALSE;
+        }
+
+        pszUnescapedContents = 
+            CPLUnescapeString( pszEscapedContents, &nContentLength,
+                               CPLES_BackslashQuotable );
+
+        if( !NITFWriteTRE( pachUDIDL, pachTRE,
+                           pnOffset, nBytesAvailable, 
+                           pszTREName, pszUnescapedContents, 
+                           nContentLength ) )
+            return FALSE;
+        
+        CPLFree( pszTREName );
+        CPLFree( pszUnescapedContents );
+        nBytesAvailable -= (nContentLength + 14);
+    }
+
+    return TRUE;
+}
+
 /************************************************************************/
 /*                          NITFWriteBLOCKA()                           */
 /************************************************************************/
 
-static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE, int nTRECount,
+static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE,
                             int *pnOffset, int nBytesAvailable,
                             char **papszOptions )
 
@@ -699,32 +853,9 @@ static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE, int nTRECount,
 /* ==================================================================== */
     for( iBlock = 1; iBlock <= nBlockCount; iBlock++ )
     {
-        char szTemp[200];
-        int iField, nOldOffset;
+        char szBLOCKA[200];
+        int iField;
 
-/* -------------------------------------------------------------------- */
-/*      Try to allocate space in the header for for this BLOCKA         */
-/*      TRE.                                                            */
-/* -------------------------------------------------------------------- */
-        if( nBytesAvailable < 137 )
-        {
-            CPLError( CE_Warning, CPLE_AppDefined,
-                      "Some BLOCKA TREs not written due to lack of header space." );
-            return nTRECount;
-        }
-
-/* -------------------------------------------------------------------- */
-/*      Update IXSHDL.                                                  */
-/* -------------------------------------------------------------------- */
-        nOldOffset = atoi(NITFGetField( szTemp, pachUDIDL, 5, 5 ));
-        sprintf( szTemp, "%05d", nOldOffset + 137 );
-        PLACE( pachUDIDL + 5, IXSHDL, szTemp );
-
-/* -------------------------------------------------------------------- */
-/*      Create TRE header.                                              */
-/* -------------------------------------------------------------------- */
-        sprintf( pachTRE + nOldOffset, "%03dBLOCKA00123", nTRECount );
-        
 /* -------------------------------------------------------------------- */
 /*      Write all fields.                                               */
 /* -------------------------------------------------------------------- */
@@ -742,24 +873,23 @@ static int NITFWriteBLOCKA( char *pachUDIDL, char *pachTRE, int nTRECount,
             if( pszValue == NULL )
                 pszValue = "";
 
-            memset( pachTRE + nOldOffset + 14 + iStart, ' ', iSize );
-            memcpy( pachTRE + nOldOffset + 14 + iStart 
-                    + MAX(0,iSize-strlen(pszValue)), 
+            memset( szBLOCKA + iStart, ' ', iSize );
+            memcpy( szBLOCKA + iStart + MAX(0,iSize-strlen(pszValue)), 
                     pszValue, MIN(iSize,strlen(pszValue)) );
         }
 
         // required field - semantics unknown. 
-        memcpy( pachTRE + nOldOffset + 14 + 118, "010.0", 5);
+        memcpy( szBLOCKA + 118, "010.0", 5);
 
-/* -------------------------------------------------------------------- */
-/*      Increment values.                                               */
-/* -------------------------------------------------------------------- */
-        nTRECount++;
-        nBytesAvailable -= 137;
-        *pnOffset += 137;
+        if( !NITFWriteTRE( pachUDIDL, pachTRE, 
+                           pnOffset, nBytesAvailable, 
+                           "BLOCKA", szBLOCKA, 123 ) )
+            return FALSE;
+
+        nBytesAvailable -= (123 + 14);
     }
     
-    return nTRECount;
+    return TRUE;
 }
                       
 /************************************************************************/
