@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: rpftocfile.c rouault $
+ * $Id: rpftocfile.cpp 14971 2008-07-19 12:57:18Z rouault $
  *
  * Project:  RPF A.TOC read Library
  * Purpose:  Module responsible for opening a RPF TOC file, populating RPFToc
@@ -49,7 +49,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: rpftocfile.c rouault $");
+CPL_CVSID("$Id: rpftocfile.cpp 14971 2008-07-19 12:57:18Z rouault $");
 
 /************************************************************************/
 /*                        RPFTOCTrim()                                    */
@@ -317,7 +317,7 @@ RPFToc* RPFTOCReadFromBuffer(const char* pszFilename, FILE* fp, const char* tocH
         CPL_MSBPTR32( &toc->entries[i].nHorizFrames );
         
         toc->entries[i].frameEntries = (RPFTocFrameEntry*)
-                VSIMalloc(toc->entries[i].nVertFrames * toc->entries[i].nHorizFrames * sizeof(RPFTocFrameEntry));
+                VSIMalloc3(toc->entries[i].nVertFrames, toc->entries[i].nHorizFrames, sizeof(RPFTocFrameEntry));
         if (toc->entries[i].frameEntries == NULL)
         {
             CPLError( CE_Failure, CPLE_OutOfMemory,
