@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: btdataset.cpp 13727 2008-02-08 01:02:30Z rayg $
+ * $Id: btdataset.cpp 12232 2007-09-23 18:13:59Z rouault $
  *
  * Project:  VTP .bt Driver
  * Purpose:  Implementation of VTP .bt elevation format read/write support.
@@ -31,7 +31,7 @@
 #include "rawdataset.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: btdataset.cpp 13727 2008-02-08 01:02:30Z rayg $");
+CPL_CVSID("$Id: btdataset.cpp 12232 2007-09-23 18:13:59Z rouault $");
 
 CPL_C_START
 void    GDALRegister_BT(void);
@@ -417,12 +417,6 @@ CPLErr BTDataset::SetGeoTransform( double *padfTransform )
     dfTop = adfGeoTransform[3];
     dfBottom = dfTop + adfGeoTransform[5] * nRasterYSize;
 
-    // rcg, feb 7/08, pixel-as-point fix:
-    dfLeft   += adfGeoTransform[1] / 2;
-    dfRight  -= adfGeoTransform[1] / 2;
-    dfTop    += adfGeoTransform[5] / 2;
-    dfBottom -= adfGeoTransform[5] / 2;
-
     memcpy( abyHeader + 28, &dfLeft, 8 );
     memcpy( abyHeader + 36, &dfRight, 8 );
     memcpy( abyHeader + 44, &dfBottom, 8 );
@@ -606,6 +600,7 @@ GDALDataset *BTDataset::Open( GDALOpenInfo * poOpenInfo )
     /*
         rcg, apr 7/06: read offset 62 for vert. units.
         If zero, assume 1.0 as per spec.
+
     */
     memcpy( &poDS->m_fVscale, poDS->abyHeader + 62, 4 );
     CPL_LSBPTR32(&poDS->m_fVscale);
@@ -744,16 +739,12 @@ GDALDataset *BTDataset::Open( GDALOpenInfo * poOpenInfo )
         memcpy( &dfTop, poDS->abyHeader + 52, 8 );
         CPL_LSBPTR64( &dfTop );
 
-        // rcg, feb 7/08, pixel-as-point fix.
         poDS->adfGeoTransform[0] = dfLeft;
-        poDS->adfGeoTransform[1] = (dfRight - dfLeft) / (poDS->nRasterXSize - 1);
+        poDS->adfGeoTransform[1] = (dfRight - dfLeft) / poDS->nRasterXSize;
         poDS->adfGeoTransform[2] = 0.0;
         poDS->adfGeoTransform[3] = dfTop;
         poDS->adfGeoTransform[4] = 0.0;
-        poDS->adfGeoTransform[5] = (dfBottom - dfTop) / (poDS->nRasterYSize - 1);
-
-        poDS->adfGeoTransform[0] -= poDS->adfGeoTransform[1] / 2;
-        poDS->adfGeoTransform[3] -= poDS->adfGeoTransform[5] / 2;
+        poDS->adfGeoTransform[5] = (dfBottom - dfTop) / poDS->nRasterYSize;
         
         poDS->bGeoTransformValid = TRUE;
     }
