@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: usgsdem_create.cpp 12490 2007-10-22 13:53:32Z dron $
+ * $Id: usgsdem_create.cpp 14522 2008-05-25 13:28:11Z rouault $
  *
  * Project:  USGS DEM Driver
  * Purpose:  CreateCopy() implementation.
@@ -36,7 +36,7 @@
 #include "gdalwarper.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: usgsdem_create.cpp 12490 2007-10-22 13:53:32Z dron $");
+CPL_CVSID("$Id: usgsdem_create.cpp 14522 2008-05-25 13:28:11Z rouault $");
 
 typedef struct 
 {
@@ -97,7 +97,17 @@ const char *USGSDEMDecToPackedDMS( double dfDec )
     nSign = ( dfDec < 0.0 )? -1 : 1;
 
     dfDec = ABS( dfDec );
-    nDegrees = (int) floor( dfDec );
+    /* If the difference between the value and the nearest degree
+       is less than 1e-5 second, then we force to round to the
+       nearest degree, to avoid result strings like '40 59 60.0000' instead of '41'.
+       This is of general interest, but was mainly done to workaround a strange
+       Valgrind bug when running usgsdem_6 where the value of psDInfo->dfULCornerY
+       computed in DTEDOpen() differ between Valgrind and non-Valgrind executions.
+    */
+    if (fabs(dfDec - (int) floor( dfDec + .5)) < 1e-5 / 3600)
+        dfDec = nDegrees = (int) floor( dfDec + .5);
+    else
+        nDegrees = (int) floor( dfDec );
     nMinutes = (int) floor( ( dfDec - nDegrees ) * 60.0 );
     dfSeconds = (dfDec - nDegrees) * 3600.0 - nMinutes * 60.0;
 
@@ -1268,7 +1278,7 @@ static int USGSDEMLoadRaster( USGSDEMWriteInfo *psWInfo,
 /*      Allocate output array, and pre-initialize to NODATA value.      */
 /* -------------------------------------------------------------------- */
     psWInfo->panData = 
-        (GInt16 *) VSIMalloc( 2 * psWInfo->nXSize * psWInfo->nYSize );
+        (GInt16 *) VSIMalloc3( 2, psWInfo->nXSize, psWInfo->nYSize );
     if( psWInfo->panData == NULL )
     {
         CPLError( CE_Failure, CPLE_OutOfMemory, 
@@ -1412,6 +1422,13 @@ USGSDEMCreateCopy( const char *pszFilename, GDALDataset *poSrcDS,
     sWInfo.utmzone = 0;
     strncpy( sWInfo.horizdatum, "", 1 );
 
+    if ( sWInfo.nXSize <= 1 || sWInfo.nYSize <= 1 )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Source dataset dimensions must be at least 2x2." );
+        return NULL;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Work out corner coordinates.                                    */
 /* -------------------------------------------------------------------- */
