@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: xpmdataset.cpp 10645 2007-01-18 02:22:39Z warmerdam $
+ * $Id: xpmdataset.cpp 15202 2008-08-23 21:42:12Z rouault $
  *
  * Project:  XPM Driver
  * Purpose:  Implement GDAL XPM Support
@@ -33,7 +33,7 @@
 #include "gdal_frmts.h"						      
 
 
-CPL_CVSID("$Id: xpmdataset.cpp 10645 2007-01-18 02:22:39Z warmerdam $");
+CPL_CVSID("$Id: xpmdataset.cpp 15202 2008-08-23 21:42:12Z rouault $");
 
 static unsigned char *ParseXPM( const char *pszInput,
                                 int *pnXSize, int *pnYSize, 
@@ -100,6 +100,9 @@ GDALDataset *XPMDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
+    if (poOpenInfo->fp == NULL)
+        return NULL;
+
 /* -------------------------------------------------------------------- */
 /*      Read the whole file into a memory strings.                      */
 /* -------------------------------------------------------------------- */
@@ -583,7 +586,7 @@ ParseXPM( const char *pszInput, int *pnXSize, int *pnYSize,
 /* -------------------------------------------------------------------- */
     GByte *pabyImage;
 
-    pabyImage = (GByte *) VSIMalloc(*pnXSize * *pnYSize);
+    pabyImage = (GByte *) VSIMalloc2(*pnXSize, *pnYSize);
     if( pabyImage == NULL )
     {
         CPLError( CE_Failure, CPLE_OutOfMemory, 
