@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalpamdataset.cpp 15069 2008-07-29 06:06:33Z warmerdam $
+ * $Id: gdalpamdataset.cpp 15033 2008-07-25 22:35:50Z warmerdam $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALPamDataset, a dataset base class that 
@@ -32,7 +32,7 @@
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: gdalpamdataset.cpp 15069 2008-07-29 06:06:33Z warmerdam $");
+CPL_CVSID("$Id: gdalpamdataset.cpp 15033 2008-07-25 22:35:50Z warmerdam $");
 
 /************************************************************************/
 /*                           GDALPamDataset()                           */
@@ -860,6 +860,15 @@ CPLErr GDALPamDataset::CloneInfo( GDALDataset *poSrcDS, int nCloneFlags )
                 SetMetadata( poSrcDS->GetMetadata() );
             }
         }
+        if( poSrcDS->GetMetadata("RPC") != NULL )
+        {
+            if( !bOnlyIfMissing 
+                || CSLCount(GetMetadata("RPC")) 
+                   != CSLCount(poSrcDS->GetMetadata("RPC")) )
+            {
+                SetMetadata( poSrcDS->GetMetadata("RPC"), "RPC" );
+            }
+        }
     }
 
 /* -------------------------------------------------------------------- */
