@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_usgs.cpp 13064 2007-11-26 15:15:42Z dron $
+ * $Id: ogr_srs_usgs.cpp 15826 2008-11-27 22:28:25Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation to/from USGS georeferencing
@@ -32,7 +32,7 @@
 #include "cpl_conv.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_usgs.cpp 13064 2007-11-26 15:15:42Z dron $");
+CPL_CVSID("$Id: ogr_srs_usgs.cpp 15826 2008-11-27 22:28:25Z warmerdam $");
 
 /************************************************************************/
 /*  GCTP projection codes.                                              */
@@ -141,7 +141,7 @@ static const long aoEllips[] =
     0// FIXME: WGS 60 --- skipped
 };
 
-#define NUMBER_OF_ELLIPSOIDS    (sizeof(aoEllips)/sizeof(aoEllips[0]))
+#define NUMBER_OF_ELLIPSOIDS    (int)(sizeof(aoEllips)/sizeof(aoEllips[0]))
 
 /************************************************************************/
 /*                        USGSGetUOMLengthInfo()                        */
@@ -312,6 +312,11 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
                                                            iDatum );
 }
 
+static double OGRSpatialReferenceUSGSUnpackNoOp(double dfVal)
+{
+    return dfVal;
+}
+
 /************************************************************************/
 /*                          importFromUSGS()                            */
 /************************************************************************/
@@ -323,9 +328,11 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
  * software. GCTP operates on angles in packed DMS format (see
  * CPLDecToPackedDMS() function for details), so all angle values (latitudes,
  * longitudes, azimuths, etc.) specified in the padfPrjParams array should
- * be in the packed DMS format.
+ * be in the packed DMS format, unless bAnglesInPackedDMSFormat is set to FALSE.
  *
  * This function is the equivalent of the C function OSRImportFromUSGS().
+ * Note that the bAnglesInPackedDMSFormat parameter is only present in the C++
+ * method. The C function assumes bAnglesInPackedFormat = TRUE.
  *
  * @param iProjSys Input projection system code, used in GCTP.
  *
@@ -477,7 +484,7 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
  * fields are set to zero too.
  * </pre>
  *
- * @param iDatum Output spheroid.<p>
+ * @param iDatum Input spheroid.<p>
  *
  * If the datum code is negative, the first two values in the parameter array
  * (parm) are used to define the values as follows:
@@ -543,12 +550,18 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
 
 OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                                             double *padfPrjParams,
-                                            long iDatum )
+                                            long iDatum, int bAnglesInPackedDMSFormat )
 
 {
     if( !padfPrjParams )
         return OGRERR_CORRUPT_DATA;
 
+    double (*pfnUnpackAnglesFn)(double);
+    if (bAnglesInPackedDMSFormat)
+        pfnUnpackAnglesFn = CPLPackedDMSToDec;
+    else
+        pfnUnpackAnglesFn = OGRSpatialReferenceUSGSUnpackNoOp;
+
 /* -------------------------------------------------------------------- */
 /*      Operate on the basis of the projection code.                    */
 /* -------------------------------------------------------------------- */
@@ -567,9 +580,9 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                         iZone = (long) padfPrjParams[2];
                     else if (padfPrjParams[0] != 0.0 && padfPrjParams[1] != 0.0)
                     {
-                        iZone = (long)(((CPLPackedDMSToDec(padfPrjParams[0])
+                        iZone = (long)(((pfnUnpackAnglesFn(padfPrjParams[0])
                                          + 180.0) / 6.0) + 1.0);
-                        if ( CPLPackedDMSToDec(padfPrjParams[0]) < 0 )
+                        if ( pfnUnpackAnglesFn(padfPrjParams[0]) < 0 )
                             bNorth = FALSE;
                     }
                 }
@@ -592,153 +605,154 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                 else if ( iDatum != 8 )
                     CPLError( CE_Warning, CPLE_AppDefined,
                               "Wrong datum for State Plane projection %d. "
-                              "Should be 0 or 8.", iDatum );
+                              "Should be 0 or 8.", (int) iDatum );
                 
                 SetStatePlane( iZone, bNAD83 );
             }
             break;
 
         case ALBERS:
-            SetACEA( CPLPackedDMSToDec(padfPrjParams[2]),
-                     CPLPackedDMSToDec(padfPrjParams[3]),
-                     CPLPackedDMSToDec(padfPrjParams[5]),
-                     CPLPackedDMSToDec(padfPrjParams[4]),
+            SetACEA( pfnUnpackAnglesFn(padfPrjParams[2]),
+                     pfnUnpackAnglesFn(padfPrjParams[3]),
+                     pfnUnpackAnglesFn(padfPrjParams[5]),
+                     pfnUnpackAnglesFn(padfPrjParams[4]),
                      padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case LAMCC:
-            SetLCC( CPLPackedDMSToDec(padfPrjParams[2]),
-                    CPLPackedDMSToDec(padfPrjParams[3]),
-                    CPLPackedDMSToDec(padfPrjParams[5]),
-                    CPLPackedDMSToDec(padfPrjParams[4]),
+            SetLCC( pfnUnpackAnglesFn(padfPrjParams[2]),
+                    pfnUnpackAnglesFn(padfPrjParams[3]),
+                    pfnUnpackAnglesFn(padfPrjParams[5]),
+                    pfnUnpackAnglesFn(padfPrjParams[4]),
                     padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case MERCAT:
-            SetMercator( CPLPackedDMSToDec(padfPrjParams[5]),
-                         CPLPackedDMSToDec(padfPrjParams[4]),
+            SetMercator( pfnUnpackAnglesFn(padfPrjParams[5]),
+                         pfnUnpackAnglesFn(padfPrjParams[4]),
                          1.0,
                          padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case PS:
-            SetPS( CPLPackedDMSToDec(padfPrjParams[5]),
-                   CPLPackedDMSToDec(padfPrjParams[4]),
+            SetPS( pfnUnpackAnglesFn(padfPrjParams[5]),
+                   pfnUnpackAnglesFn(padfPrjParams[4]),
                    1.0,
                    padfPrjParams[6], padfPrjParams[7] );
 
             break;
 
         case POLYC:
-            SetPolyconic( CPLPackedDMSToDec(padfPrjParams[5]),
-                          CPLPackedDMSToDec(padfPrjParams[4]),
+            SetPolyconic( pfnUnpackAnglesFn(padfPrjParams[5]),
+                          pfnUnpackAnglesFn(padfPrjParams[4]),
                           padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case EQUIDC:
             if ( padfPrjParams[8] )
             {
-                SetEC( CPLPackedDMSToDec(padfPrjParams[2]),
-                       CPLPackedDMSToDec(padfPrjParams[3]),
-                       CPLPackedDMSToDec(padfPrjParams[5]),
-                       CPLPackedDMSToDec(padfPrjParams[4]),
+                SetEC( pfnUnpackAnglesFn(padfPrjParams[2]),
+                       pfnUnpackAnglesFn(padfPrjParams[3]),
+                       pfnUnpackAnglesFn(padfPrjParams[5]),
+                       pfnUnpackAnglesFn(padfPrjParams[4]),
                        padfPrjParams[6], padfPrjParams[7] );
             }
             else
             {
-                SetEC( CPLPackedDMSToDec(padfPrjParams[2]),
-                       CPLPackedDMSToDec(padfPrjParams[2]),
-                       CPLPackedDMSToDec(padfPrjParams[5]),
-                       CPLPackedDMSToDec(padfPrjParams[4]),
+                SetEC( pfnUnpackAnglesFn(padfPrjParams[2]),
+                       pfnUnpackAnglesFn(padfPrjParams[2]),
+                       pfnUnpackAnglesFn(padfPrjParams[5]),
+                       pfnUnpackAnglesFn(padfPrjParams[4]),
                        padfPrjParams[6], padfPrjParams[7] );
             }
             break;
 
         case TM:
-            SetTM( CPLPackedDMSToDec(padfPrjParams[5]),
-                   CPLPackedDMSToDec(padfPrjParams[4]),
+            SetTM( pfnUnpackAnglesFn(padfPrjParams[5]),
+                   pfnUnpackAnglesFn(padfPrjParams[4]),
                    padfPrjParams[2],
                    padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case STEREO:
-            SetStereographic( CPLPackedDMSToDec(padfPrjParams[5]),
-                              CPLPackedDMSToDec(padfPrjParams[4]),
+            SetStereographic( pfnUnpackAnglesFn(padfPrjParams[5]),
+                              pfnUnpackAnglesFn(padfPrjParams[4]),
                               1.0,
                               padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case LAMAZ:
-            SetLAEA( CPLPackedDMSToDec(padfPrjParams[5]),
-                     CPLPackedDMSToDec(padfPrjParams[4]),
+            SetLAEA( pfnUnpackAnglesFn(padfPrjParams[5]),
+                     pfnUnpackAnglesFn(padfPrjParams[4]),
                      padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case AZMEQD:
-            SetAE( CPLPackedDMSToDec(padfPrjParams[5]),
-                   CPLPackedDMSToDec(padfPrjParams[4]),
+            SetAE( pfnUnpackAnglesFn(padfPrjParams[5]),
+                   pfnUnpackAnglesFn(padfPrjParams[4]),
                    padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case GNOMON:
-            SetGnomonic( CPLPackedDMSToDec(padfPrjParams[5]),
-                         CPLPackedDMSToDec(padfPrjParams[4]),
+            SetGnomonic( pfnUnpackAnglesFn(padfPrjParams[5]),
+                         pfnUnpackAnglesFn(padfPrjParams[4]),
                          padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case ORTHO:
-            SetOrthographic( CPLPackedDMSToDec(padfPrjParams[5]),
-                             CPLPackedDMSToDec(padfPrjParams[4]),
+            SetOrthographic( pfnUnpackAnglesFn(padfPrjParams[5]),
+                             pfnUnpackAnglesFn(padfPrjParams[4]),
                              padfPrjParams[6], padfPrjParams[7] );
             break;
 
         // FIXME: GVNSP --- General Vertical Near-Side Perspective skipped
 
         case SNSOID:
-            SetSinusoidal( CPLPackedDMSToDec(padfPrjParams[4]),
+            SetSinusoidal( pfnUnpackAnglesFn(padfPrjParams[4]),
                            padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case EQRECT:
-            SetEquirectangular( CPLPackedDMSToDec(padfPrjParams[5]),
-                                CPLPackedDMSToDec(padfPrjParams[4]),
-                                padfPrjParams[6], padfPrjParams[7] );
+            SetEquirectangular2( 0.0,
+                                 pfnUnpackAnglesFn(padfPrjParams[4]),
+                                 pfnUnpackAnglesFn(padfPrjParams[5]),
+                                 padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case MILLER:
-            SetMC( CPLPackedDMSToDec(padfPrjParams[5]),
-                   CPLPackedDMSToDec(padfPrjParams[4]),
+            SetMC( pfnUnpackAnglesFn(padfPrjParams[5]),
+                   pfnUnpackAnglesFn(padfPrjParams[4]),
                    padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case VGRINT:
-            SetVDG( CPLPackedDMSToDec(padfPrjParams[4]),
+            SetVDG( pfnUnpackAnglesFn(padfPrjParams[4]),
                     padfPrjParams[6], padfPrjParams[7] );
             break;
 
         case HOM:
             if ( padfPrjParams[12] )
             {
-                SetHOM( CPLPackedDMSToDec(padfPrjParams[5]),
-                        CPLPackedDMSToDec(padfPrjParams[4]),
-                        CPLPackedDMSToDec(padfPrjParams[3]),
+                SetHOM( pfnUnpackAnglesFn(padfPrjParams[5]),
+                        pfnUnpackAnglesFn(padfPrjParams[4]),
+                        pfnUnpackAnglesFn(padfPrjParams[3]),
                         0.0, padfPrjParams[2],
                         padfPrjParams[6],  padfPrjParams[7] );
             }
             else
             {
-                SetHOM2PNO( CPLPackedDMSToDec(padfPrjParams[5]),
-                            CPLPackedDMSToDec(padfPrjParams[9]),
-                            CPLPackedDMSToDec(padfPrjParams[8]),
-                            CPLPackedDMSToDec(padfPrjParams[11]),
-                            CPLPackedDMSToDec(padfPrjParams[10]),
+                SetHOM2PNO( pfnUnpackAnglesFn(padfPrjParams[5]),
+                            pfnUnpackAnglesFn(padfPrjParams[9]),
+                            pfnUnpackAnglesFn(padfPrjParams[8]),
+                            pfnUnpackAnglesFn(padfPrjParams[11]),
+                            pfnUnpackAnglesFn(padfPrjParams[10]),
                             padfPrjParams[2],
                             padfPrjParams[6],  padfPrjParams[7] );
             }
             break;
 
         case ROBIN:
-            SetRobinson( CPLPackedDMSToDec(padfPrjParams[4]),
+            SetRobinson( pfnUnpackAnglesFn(padfPrjParams[4]),
                          padfPrjParams[6], padfPrjParams[7] );
             break;
 
@@ -749,7 +763,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
         // FIXME: GOODE --- Interrupted Goode skipped
 
         case MOLL:
-            SetMollweide( CPLPackedDMSToDec(padfPrjParams[4]),
+            SetMollweide( pfnUnpackAnglesFn(padfPrjParams[4]),
                           padfPrjParams[6], padfPrjParams[7] );
             break;
 
@@ -757,9 +771,13 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
 
         // FIXME: HAMMER --- Hammer skipped
 
-        // FIXME: WAGIV --- Wagner IV skipped
+        case WAGIV:
+            SetWagner( 4, 0.0, padfPrjParams[6], padfPrjParams[7] );
+            break;
 
-        // FIXME: WAGVII --- Wagner VII skipped
+        case WAGVII:
+            SetWagner( 7, 0.0, padfPrjParams[6], padfPrjParams[7] );
+            break;
 
         // FIXME: OBEQA --- Oblated Equal Area skipped
 
@@ -772,8 +790,8 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
         // FIXME: ISINUS --- Integrized Sinusoidal skipped
 
         default:
-            CPLDebug( "OSR_USGS", "Unsupported projection: %d", iProjSys );
-            SetLocalCS( CPLString().Printf("GCTP projection number %d", iProjSys) );
+            CPLDebug( "OSR_USGS", "Unsupported projection: %ld", iProjSys );
+            SetLocalCS( CPLString().Printf("GCTP projection number %ld", iProjSys) );
             break;
             
     }
@@ -843,7 +861,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             }
 
         }
-        else if ( iDatum < (int) NUMBER_OF_ELLIPSOIDS && aoEllips[iDatum] )
+        else if ( iDatum < NUMBER_OF_ELLIPSOIDS && aoEllips[iDatum] )
         {
             if( USGSGetEllipsoidInfo( aoEllips[iDatum], &pszName,
                                        &dfSemiMajor, &dfInvFlattening ) )
@@ -861,7 +879,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
                 CPLError( CE_Warning, CPLE_AppDefined,
                           "Failed to lookup datum code %d, likely due to missing GDAL gcs.csv\n"
                           " file.  Falling back to use WGS84.", 
-                          iDatum );
+                          (int) iDatum );
                 SetWellKnownGeogCS("WGS84" );
             }
         }
@@ -870,7 +888,7 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
             CPLError( CE_Warning, CPLE_AppDefined,
                       "Wrong datum code %d. Supported datums 0--%d only.\n"
                       "Setting WGS84 as a fallback.",
-                      iDatum, NUMBER_OF_ELLIPSOIDS );
+                      (int) iDatum, NUMBER_OF_ELLIPSOIDS );
             SetWellKnownGeogCS( "WGS84" );
         }
 
@@ -1138,7 +1156,7 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
         (*ppadfPrjParams)[4] = CPLDecToPackedDMS(
             GetNormProjParm( SRS_PP_CENTRAL_MERIDIAN, 0.0 ) );
         (*ppadfPrjParams)[5] = CPLDecToPackedDMS(
-            GetNormProjParm( SRS_PP_LATITUDE_OF_ORIGIN, 0.0 ) );
+            GetNormProjParm( SRS_PP_STANDARD_PARALLEL_1, 0.0 ) );
         (*ppadfPrjParams)[6] = GetNormProjParm( SRS_PP_FALSE_EASTING, 0.0 );
         (*ppadfPrjParams)[7] = GetNormProjParm( SRS_PP_FALSE_NORTHING, 0.0 );
     }
@@ -1216,6 +1234,20 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
         (*ppadfPrjParams)[7] = GetNormProjParm( SRS_PP_FALSE_NORTHING, 0.0 );
     }
 
+    else if( EQUAL(pszProjection, SRS_PT_WAGNER_IV) )
+    {
+        *piProjSys = WAGIV;
+        (*ppadfPrjParams)[6] = GetNormProjParm( SRS_PP_FALSE_EASTING, 0.0 );
+        (*ppadfPrjParams)[7] = GetNormProjParm( SRS_PP_FALSE_NORTHING, 0.0 );
+    }
+
+    else if( EQUAL(pszProjection, SRS_PT_WAGNER_VII) )
+    {
+        *piProjSys = WAGVII;
+        (*ppadfPrjParams)[6] = GetNormProjParm( SRS_PP_FALSE_EASTING, 0.0 );
+        (*ppadfPrjParams)[7] = GetNormProjParm( SRS_PP_FALSE_NORTHING, 0.0 );
+    }
+
     // Projection unsupported by GCTP
     else
     {
@@ -1253,7 +1285,7 @@ OGRErr OGRSpatialReference::exportToUSGS( long *piProjSys, long *piZone,
                       "Try to translate ellipsoid definition.", pszDatum );
 #endif
             
-            for ( i = 0; i < (int) NUMBER_OF_ELLIPSOIDS; i++ )
+            for ( i = 0; i < NUMBER_OF_ELLIPSOIDS; i++ )
             {
                 double  dfSM;
                 double  dfIF;
