@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrfeaturequery.cpp 11897 2007-08-17 21:26:29Z rouault $
+ * $Id: ogrfeaturequery.cpp 14288 2008-04-13 15:56:22Z rouault $
  * 
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of simple SQL WHERE style attributes queries
@@ -33,15 +33,15 @@
 #include "ogr_p.h"
 #include "ogr_attrind.h"
 
-CPL_CVSID("$Id: ogrfeaturequery.cpp 11897 2007-08-17 21:26:29Z rouault $");
+CPL_CVSID("$Id: ogrfeaturequery.cpp 14288 2008-04-13 15:56:22Z rouault $");
 
 /************************************************************************/
 /*     Support for special attributes (feature query and selection)     */
 /************************************************************************/
 
-char* SpecialFieldNames[SPECIAL_FIELD_COUNT] 
+const char* SpecialFieldNames[SPECIAL_FIELD_COUNT] 
     = {"FID", "OGR_GEOMETRY", "OGR_STYLE", "OGR_GEOM_WKT"};
-swq_field_type SpecialFieldTypes[SPECIAL_FIELD_COUNT] 
+const swq_field_type SpecialFieldTypes[SPECIAL_FIELD_COUNT] 
     = {SWQ_INTEGER, SWQ_STRING, SWQ_STRING, SWQ_STRING};
 
 /************************************************************************/
@@ -122,7 +122,7 @@ OGRErr OGRFeatureQuery::Compile( OGRFeatureDefn *poDefn,
     iField = 0;
     while (iField < SPECIAL_FIELD_COUNT)
     {
-        papszFieldNames[poDefn->GetFieldCount() + iField] = SpecialFieldNames[iField];
+        papszFieldNames[poDefn->GetFieldCount() + iField] = (char *) SpecialFieldNames[iField];
         paeFieldTypes[poDefn->GetFieldCount() + iField] = SpecialFieldTypes[iField];
         ++iField;
     }
