@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeometry.cpp 13334 2007-12-13 19:23:21Z warmerdam $
+ * $Id: ogrgeometry.cpp 15346 2008-09-08 18:28:46Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements a few base methods on OGRGeometry.
@@ -34,7 +34,7 @@
 #include "cpl_multiproc.h"
 #include <assert.h>
 
-CPL_CVSID("$Id: ogrgeometry.cpp 13334 2007-12-13 19:23:21Z warmerdam $");
+CPL_CVSID("$Id: ogrgeometry.cpp 15346 2008-09-08 18:28:46Z rouault $");
 
 int OGRGeometry::bGenerate_DB2_V72_BYTE_ORDER = FALSE;
 
@@ -94,7 +94,7 @@ OGRGeometry::~OGRGeometry()
  * @param pszPrefix the prefix to put on each line of output.
  */
 
-void OGRGeometry::dumpReadable( FILE * fp, const char * pszPrefix ) const
+void OGRGeometry::dumpReadable( FILE * fp, const char * pszPrefix, char** papszOptions ) const
 
 {
     char        *pszWkt = NULL;
@@ -105,10 +105,84 @@ void OGRGeometry::dumpReadable( FILE * fp, const char * pszPrefix ) const
     if( fp == NULL )
         fp = stdout;
 
-    if( exportToWkt( &pszWkt ) == OGRERR_NONE )
+    const char* pszDisplayGeometry =
+                CSLFetchNameValue(papszOptions, "DISPLAY_GEOMETRY");
+    if (pszDisplayGeometry != NULL && EQUAL(pszDisplayGeometry, "SUMMARY"))
     {
-        fprintf( fp, "%s%s\n", pszPrefix, pszWkt );
-        CPLFree( pszWkt );
+        OGRLineString *poLine;
+        OGRPolygon *poPoly;
+        OGRLinearRing *poRing;
+        OGRGeometryCollection *poColl;
+        fprintf( fp, "%s%s : ", pszPrefix, getGeometryName() );
+        switch( getGeometryType() )
+        {
+            case wkbUnknown:
+            case wkbNone:
+                break;
+            case wkbPoint:
+            case wkbPoint25D:
+                break;
+            case wkbLineString:
+            case wkbLineString25D:
+                poLine = (OGRLineString*)this;
+                fprintf( fp, "%d points\n", poLine->getNumPoints() );
+                break;
+            case wkbPolygon:
+            case wkbPolygon25D:
+            {
+                int ir;
+                int nRings;
+                poPoly = (OGRPolygon*)this;
+                poRing = poPoly->getExteriorRing();
+                nRings = poPoly->getNumInteriorRings();
+                fprintf( fp, "%d points", poRing->getNumPoints() );
+                if (nRings)
+                {
+                    fprintf( fp, ", %d inner rings (", nRings);
+                    for( ir = 0; ir < nRings; ir++)
+                    {
+                        if (ir)
+                            fprintf( fp, ", ");
+                        fprintf( fp, "%d points",
+                                 poPoly->getInteriorRing(ir)->getNumPoints() );
+                    }
+                    fprintf( fp, ")");
+                }
+                fprintf( fp, "\n");
+                break;
+            }
+            case wkbMultiPoint:
+            case wkbMultiPoint25D:
+            case wkbMultiLineString:
+            case wkbMultiLineString25D:
+            case wkbMultiPolygon:
+            case wkbMultiPolygon25D:
+            case wkbGeometryCollection:
+            case wkbGeometryCollection25D:
+            {
+                int ig;
+                poColl = (OGRGeometryCollection*)this;
+                fprintf( fp, "%d geometries:\n", poColl->getNumGeometries() );
+                for ( ig = 0; ig < poColl->getNumGeometries(); ig++)
+                {
+                    OGRGeometry * poChild = (OGRGeometry*)poColl->getGeometryRef(ig);
+                    fprintf( fp, "%s", pszPrefix);
+                    poChild->dumpReadable( fp, pszPrefix, papszOptions );
+                }
+                break;
+            }
+            case wkbLinearRing:
+                break;
+        }
+    }
+    else if (pszDisplayGeometry == NULL || CSLTestBoolean(pszDisplayGeometry) ||
+             EQUAL(pszDisplayGeometry, "WKT"))
+    {
+        if( exportToWkt( &pszWkt ) == OGRERR_NONE )
+        {
+            fprintf( fp, "%s%s\n", pszPrefix, pszWkt );
+            CPLFree( pszWkt );
+        }
     }
 }
 
@@ -465,6 +539,54 @@ OGRErr OGR_G_Transform( OGRGeometryH hGeom,
  * @return 0 for points, 1 for lines and 2 for surfaces.
  */
 
+
+/************************************************************************/
+/*                  OGRGeometry::segmentize()                           */
+/************************************************************************/
+/**
+ *
+ * Modify the geometry such it has no segment longer then the given distance.
+ * Interpolated points will have Z and M values (if needed) set to 0.
+ * Distance computation is performed in 2d only
+ *
+ * This function is the same as the C function OGR_G_Segmentize()
+ *
+ * @param hGeom handle on the geometry to segmentize
+ * @param dfMaxLength the maximum distance between 2 points after segmentization
+ */
+
+void OGRGeometry::segmentize( double dfMaxLength )
+{
+    /* Do nothing */
+}
+
+/************************************************************************/
+/*                         OGR_G_Segmentize()                           */
+/************************************************************************/
+
+/**
+ *
+ * Modify the geometry such it has no segment longer then the given distance.
+ * Interpolated points will have Z and M values (if needed) set to 0.
+ * Distance computation is performed in 2d only
+ *
+ * This function is the same as the CPP method OGRGeometry::segmentize().
+ *
+ * @param hGeom handle on the geometry to segmentize
+ * @param dfMaxLength the maximum distance between 2 points after segmentization
+ */
+
+void   CPL_DLL OGR_G_Segmentize(OGRGeometryH hGeom, double dfMaxLength )
+{
+    if (dfMaxLength <= 0)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "dfMaxLength must be strictly positive");
+        return;
+    }
+    ((OGRGeometry *) hGeom)->segmentize( dfMaxLength );
+}
+
 /************************************************************************/
 /*                         OGR_G_GetDimension()                         */
 /************************************************************************/
@@ -564,38 +686,6 @@ void OGR_G_SetCoordinateDimension( OGRGeometryH hGeom, int nNewDimension)
     ((OGRGeometry *) hGeom)->setCoordinateDimension( nNewDimension );
 }
 
-
-/**
- * \fn OGRBoolean OGRGeometry::IsEmpty() const;
- *
- * Returns TRUE (non-zero) if the object has no points.  Normally this
- * returns FALSE except between when an object is instantiated and points
- * have been assigned.
- *
- * This method relates to the SFCOM IGeometry::IsEmpty() method.
- *
- * NOTE: This method is hardcoded to return FALSE at this time.
- *
- * @return TRUE if object is empty, otherwise FALSE.
- */
-
-/**
- * \fn OGRBoolean OGRGeometry::IsSimple() const;
- *
- * Returns TRUE if the geometry is simple.
- * 
- * Returns TRUE if the geometry has no anomalous geometric points, such
- * as self intersection or self tangency. The description of each
- * instantiable geometric class will include the specific conditions that
- * cause an instance of that class to be classified as not simple.
- *
- * This method relates to the SFCOM IGeometry::IsSimple() method.
- *
- * NOTE: This method is hardcoded to return TRUE at this time.
- *
- * @return TRUE if object is simple, otherwise FALSE.
- */
-
 /**
  * \fn int OGRGeometry::Equals( OGRGeometry *poOtherGeom ) const;
  *
@@ -1087,50 +1177,30 @@ void OGR_G_Empty( OGRGeometryH hGeom )
     ((OGRGeometry *) hGeom)->empty();
 }
 
+/**
+ * \fn OGRBoolean OGRGeometry::IsEmpty() const;
+ *
+ * Returns TRUE (non-zero) if the object has no points.  Normally this
+ * returns FALSE except between when an object is instantiated and points
+ * have been assigned.
+ *
+ * This method relates to the SFCOM IGeometry::IsEmpty() method.
+ *
+ * @return TRUE if object is empty, otherwise FALSE.
+ */
+
 /************************************************************************/
-/*                              IsEmpty()                               */
+/*                         OGR_G_IsEmpty()                              */
 /************************************************************************/
 
 /**
  * Test if the geometry is empty
  *
- * This method is the same as the C function OGR_G_IsEmpty().
- *
- * This method is built on the GEOS library, check it for the definition
- * of the geometry operation.
- * If OGR is built without the GEOS library, this method will always return 
- * FALSE. 
- *
+ * This method is the same as the CPP method OGRGeometry::IsEmpty().
  *
  * @return TRUE if the geometry has no points, otherwise FALSE.  
  */
 
-OGRBoolean
-OGRGeometry::IsEmpty(  ) const
-
-{
-#ifndef HAVE_GEOS
-
-    return FALSE;
-
-#else
-
-    OGRBoolean bResult = FALSE;
-    GEOSGeom hThisGeosGeom = NULL;
-    
-    hThisGeosGeom = exportToGEOS();
-
-    if( hThisGeosGeom != NULL  )
-    {
-        bResult = GEOSisEmpty( hThisGeosGeom );
-        GEOSGeom_destroy( hThisGeosGeom );
-    }
-
-    return bResult;
-
-#endif /* HAVE_GEOS */
-}
-
 int OGR_G_IsEmpty( OGRGeometryH hGeom )
 
 {
@@ -1231,6 +1301,22 @@ OGRGeometry::IsSimple(  ) const
 #endif /* HAVE_GEOS */
 }
 
+
+/**
+ * Returns TRUE if the geometry is simple.
+ * 
+ * Returns TRUE if the geometry has no anomalous geometric points, such
+ * as self intersection or self tangency. The description of each
+ * instantiable geometric class will include the specific conditions that
+ * cause an instance of that class to be classified as not simple.
+ *
+ * This method relates to the SFCOM IGeometry::IsSimple() method.
+ *
+ * NOTE: This method is hardcoded to return TRUE at this time.
+ *
+ * @return TRUE if object is simple, otherwise FALSE.
+ */
+
 int OGR_G_IsSimple( OGRGeometryH hGeom )
 
 {
@@ -1368,6 +1454,71 @@ const char *OGRGeometryTypeToName( OGRwkbGeometryType eType )
     }
 }
 
+/************************************************************************/
+/*                       OGRMergeGeometryTypes()                        */
+/************************************************************************/
+
+/**
+ * Find common geometry type.
+ *
+ * Given two geometry types, find the most specific common
+ * type.  Normally used repeatedly with the geometries in a
+ * layer to try and establish the most specific geometry type
+ * that can be reported for the layer.
+ *
+ * NOTE: wkbUnknown is the "worst case" indicating a mixture of
+ * geometry types with nothing in common but the base geometry
+ * type.  wkbNone should be used to indicate that no geometries
+ * have been encountered yet, and means the first geometry
+ * encounted will establish the preliminary type.
+ * 
+ * @param eMain the first input geometry type.
+ * @param eExtra the second input geometry type.
+ *
+ * @return the merged geometry type.
+ */
+
+OGRwkbGeometryType 
+OGRMergeGeometryTypes( OGRwkbGeometryType eMain,
+                       OGRwkbGeometryType eExtra )
+
+{
+    int n25DFlag = 0;
+    OGRwkbGeometryType eFMain = wkbFlatten(eMain);
+    OGRwkbGeometryType eFExtra = wkbFlatten(eExtra);
+        
+    if( eFMain != eMain || eFExtra != eExtra )
+        n25DFlag = wkb25DBit;
+
+    if( eFMain == wkbUnknown || eFExtra == wkbUnknown )
+        return (OGRwkbGeometryType) (((int) wkbUnknown) | n25DFlag);
+
+    if( eFMain == wkbNone )
+        return eExtra;
+
+    if( eFExtra == wkbNone )
+        return eMain;
+
+    if( eFMain == eFExtra )
+        return (OGRwkbGeometryType) (((int) eFMain) | n25DFlag);
+
+    // Both are geometry collections.
+    if( (eFMain == wkbGeometryCollection
+         || eFMain == wkbMultiPoint
+         || eFMain == wkbMultiLineString
+         || eFMain == wkbMultiPolygon)
+        && (eFExtra == wkbGeometryCollection
+            || eFExtra == wkbMultiPoint
+            || eFExtra == wkbMultiLineString
+            || eFMain == wkbMultiPolygon) )
+    {
+        return (OGRwkbGeometryType) (((int) wkbGeometryCollection) | n25DFlag);
+    }
+
+    // Nothing apparently in common.
+    return (OGRwkbGeometryType) (((int) wkbUnknown) | n25DFlag);
+}
+
 /**
  * \fn void OGRGeometry::flattenTo2D();
  *
@@ -1436,6 +1587,7 @@ char *OGRGeometry::exportToGML() const
 
 char *OGRGeometry::exportToKML() const
 {
+#ifndef _WIN32_WCE
 #ifdef OGR_ENABLED
     return OGR_G_ExportToKML( (OGRGeometryH) this, NULL );
 #else
@@ -1443,6 +1595,11 @@ char *OGRGeometry::exportToKML() const
               "OGRGeometry::exportToKML() not supported in builds without OGR drivers." );
     return NULL;
 #endif
+#else
+    CPLError( CE_Failure, CPLE_AppDefined,
+              "OGRGeometry::exportToKML() not supported in the WinCE build." );
+    return NULL;
+#endif
 }
 
 /************************************************************************/
@@ -1463,14 +1620,20 @@ char *OGRGeometry::exportToKML() const
 
 char *OGRGeometry::exportToJson() const
 {
+#ifndef _WIN32_WCE
 #ifdef OGR_ENABLED
     OGRGeometry* poGeometry = const_cast<OGRGeometry*>(this);
-    return OGR_G_ExportToJson( static_cast<OGRGeometryH>(poGeometry) );
+    return OGR_G_ExportToJson( (OGRGeometryH) (poGeometry) );
 #else
     CPLError( CE_Failure, CPLE_AppDefined,
               "OGRGeometry::exportToJson() not supported in builds without OGR drivers." );
     return NULL;
 #endif
+#else
+    CPLError( CE_Failure, CPLE_AppDefined,
+              "OGRGeometry::exportToJson() not supported in the WinCE build." );
+    return NULL;
+#endif
 }
 
 /************************************************************************/
@@ -2506,4 +2669,3 @@ void OGR_G_CloseRings( OGRGeometryH hGeom )
 {
     ((OGRGeometry *) hGeom)->closeRings();
 }
-
