@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: reader.cpp 12582 2007-10-29 09:38:21Z pka $
+ * $Id: reader.cpp 14993 2008-07-22 18:40:40Z mloskot $
  *
  * Project:  iom - The INTERLIS Object Model
  * Purpose:  For more information, please see <http://iom.sourceforge.net>
@@ -66,7 +66,8 @@ int iom_file::readHeader(const char *model)
 
 int iom_file::readLoop(const char *filename)
 {
-	try{
+	try
+    {
 		if (!parser->parseFirst(filename, token))
 		{
 			iom_issueerr("scanFirst() failed");
@@ -78,7 +79,8 @@ int iom_file::readLoop(const char *filename)
 		}
 		parser->parseReset(token);
 	}
-	catch (const XMLException& toCatch) {
+	catch (const XMLException& toCatch)
+    {
             char* message = XMLString::transcode(toCatch.getMessage());
 			iom_issueerr(message);
             XMLString::release(&message);
