@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: kml.cpp 13040 2007-11-26 00:32:51Z mloskot $
+ * $Id: kml.cpp 14947 2008-07-17 18:16:40Z condit $
  *
  * Project:  KML Driver
  * Purpose:  Class for reading, parsing and handling a kmlfile.
@@ -39,7 +39,7 @@
 KML::KML()
 {
 	nDepth_ = 0;
-	bValid_ = false;
+	validity = KML_VALIDITY_UNKNOWN;
 	pKMLFile_ = NULL;
 	sError_ = "";
 	poTrunk_ = NULL;
@@ -50,7 +50,7 @@ KML::KML()
 KML::~KML()
 {
     if( NULL != pKMLFile_ )
-        VSIFClose(pKMLFile_);
+        VSIFCloseL(pKMLFile_);
 
     delete poTrunk_;
 }
@@ -58,9 +58,9 @@ KML::~KML()
 bool KML::open(const char * pszFilename)
 {
     if( NULL != pKMLFile_ )
-        VSIFClose( pKMLFile_ );
+        VSIFCloseL( pKMLFile_ );
 
-    pKMLFile_ = VSIFOpen( pszFilename, "r" );
+    pKMLFile_ = VSIFOpenL( pszFilename, "r" );
     if( NULL == pKMLFile_ )
     {
         return FALSE;
@@ -74,12 +74,12 @@ void KML::parse()
     std::size_t nDone = 0;
     std::size_t nLen = 0;
     char aBuf[BUFSIZ] = { 0 };
-	
-	if( NULL == pKMLFile_ )
+
+    if( NULL == pKMLFile_ )
     {
-		sError_ = "No file given";
-		return;
-	}
+        sError_ = "No file given";
+        return;
+    }
 
     if(poTrunk_ != NULL) {
         delete poTrunk_;
@@ -91,30 +91,31 @@ void KML::parse()
         delete poCurrent_;
         poCurrent_ = NULL;
     }
-	
-	XML_Parser oParser = XML_ParserCreate(NULL);
-	XML_SetUserData(oParser, this);
-	XML_SetElementHandler(oParser, startElement, endElement);
-	XML_SetCharacterDataHandler(oParser, dataHandler);
+
+    XML_Parser oParser = XML_ParserCreate(NULL);
+    XML_SetUserData(oParser, this);
+    XML_SetElementHandler(oParser, startElement, endElement);
+    XML_SetCharacterDataHandler(oParser, dataHandler);
 
     do
     {
-        nLen = (int)VSIFRead( aBuf, 1, sizeof(aBuf), pKMLFile_ );
-        nDone = nLen < sizeof(aBuf);
+        nLen = (int)VSIFReadL( aBuf, 1, sizeof(aBuf), pKMLFile_ );
+        nDone = VSIFEofL(pKMLFile_);
         if (XML_Parse(oParser, aBuf, nLen, nDone) == XML_STATUS_ERROR)
         {
-            fprintf(stderr,
-                "%s at line %u\n",
-                XML_ErrorString(XML_GetErrorCode(oParser)),
-                XML_GetCurrentLineNumber(oParser));
+            CPLError(CE_Failure, CPLE_AppDefined,
+                        "XML parsing of KML file failed : %s at line %d, column %d",
+                        XML_ErrorString(XML_GetErrorCode(oParser)),
+                        (int)XML_GetCurrentLineNumber(oParser),
+                        (int)XML_GetCurrentColumnNumber(oParser));
             XML_ParserFree(oParser);
-            VSIRewind(pKMLFile_);
+            VSIRewindL(pKMLFile_);
             return;
         }
-    } while (!nDone);
+    } while (!nDone && nLen > 0 );
 
-	XML_ParserFree(oParser);
-    VSIRewind(pKMLFile_);
+    XML_ParserFree(oParser);
+    VSIRewindL(pKMLFile_);
     poCurrent_ = NULL;
 }
 
@@ -129,37 +130,56 @@ void KML::checkValidity()
         delete poTrunk_;
         poTrunk_ = NULL;
     }
-    
+
     if(poCurrent_ != NULL)
     {
         delete poCurrent_;
         poCurrent_ = NULL;
     }
-	
-	if(pKMLFile_ == NULL)
+
+    if(pKMLFile_ == NULL)
     {
-		this->sError_ = "No file given";
-		return;
-	}
-	
+        this->sError_ = "No file given";
+        return;
+    }
+
     XML_Parser oParser = XML_ParserCreate(NULL);
     XML_SetUserData(oParser, this);
     XML_SetElementHandler(oParser, startElementValidate, NULL);
+    int nCount = 0;
 
+    /* Parses the file until we find the first element */
     do
     {
-        nLen = (int)VSIFRead( aBuf, 1, sizeof(aBuf), pKMLFile_ );
-        nDone = nLen < sizeof(aBuf);
+        nLen = (int)VSIFReadL( aBuf, 1, sizeof(aBuf), pKMLFile_ );
+        nDone = VSIFEofL(pKMLFile_);
         if (XML_Parse(oParser, aBuf, nLen, nDone) == XML_STATUS_ERROR)
         {
+            if (nLen <= BUFSIZ-1)
+                aBuf[nLen] = 0;
+            else
+                aBuf[BUFSIZ-1] = 0;
+            if (strstr(aBuf, "<?xml") && strstr(aBuf, "<kml"))
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                        "XML parsing of KML file failed : %s at line %d, column %d",
+                        XML_ErrorString(XML_GetErrorCode(oParser)),
+                        (int)XML_GetCurrentLineNumber(oParser),
+                        (int)XML_GetCurrentColumnNumber(oParser));
+            }
+
             XML_ParserFree(oParser);
-            VSIRewind(pKMLFile_);
+            VSIRewindL(pKMLFile_);
             return;
         }
-    } while (!nDone || bValid_);
-	
+
+        nCount ++;
+        /* After reading 50 * BUFSIZE bytes, and not finding whether the file */
+        /* is KML or not, we give up and fail silently */
+    } while (!nDone && nLen > 0 && validity == KML_VALIDITY_UNKNOWN && nCount < 50);
+
     XML_ParserFree(oParser);
-    VSIRewind(pKMLFile_);
+    VSIRewindL(pKMLFile_);
     poCurrent_ = NULL;
 }
 
@@ -210,43 +230,41 @@ void XMLCALL KML::startElement(void* pUserData, const char* pszName, const char*
 
 void XMLCALL KML::startElementValidate(void* pUserData, const char* pszName, const char** ppszAttr)
 {
-	int i = 0;
-	std::string* sName = new std::string(pszName);
-	std::string* sAttribute = NULL;
-	
-	if(sName->compare("kml") == 0)
-    {
-		// Check all Attributes
-		for (i = 0; ppszAttr[i]; i += 2)
-        {
-			sAttribute = new std::string(ppszAttr[i]);
-			// Find the namespace
+    int i = 0;
+    
+    if (((KML *)pUserData)->validity != KML_VALIDITY_UNKNOWN)
+        return;
 
-			if(sAttribute->compare("xmlns") == 0)
+    ((KML *)pUserData)->validity = KML_VALIDITY_INVALID;
+
+    if(strcmp(pszName, "kml") == 0)
+    {
+        // Check all Attributes
+        for (i = 0; ppszAttr[i]; i += 2)
+        {
+            // Find the namespace and determine the KML version
+            if(strcmp(ppszAttr[i], "xmlns") == 0)
             {
-			    delete sAttribute;
-				sAttribute = new std::string(ppszAttr[i + 1]);
-				// Is it KML 2.1?
-				if(sAttribute->compare("http://earth.google.com/kml/2.2") == 0)
+                // Is it KML 2.2?
+                if((strcmp(ppszAttr[i + 1], "http://earth.google.com/kml/2.2") == 0) || 
+				   (strcmp(ppszAttr[i + 1], "http://www.opengis.net/kml/2.2") == 0))
                 {
-					((KML *)pUserData)->bValid_ = true;
-					((KML *)pUserData)->sVersion_ = "2.2 (beta)";
-				}
-                else if(sAttribute->compare("http://earth.google.com/kml/2.1") == 0)
+                    ((KML *)pUserData)->validity = KML_VALIDITY_VALID;
+                    ((KML *)pUserData)->sVersion_ = "2.2";
+                }
+                else if(strcmp(ppszAttr[i + 1], "http://earth.google.com/kml/2.1") == 0)
                 {
-					((KML *)pUserData)->bValid_ = true;
-					((KML *)pUserData)->sVersion_ = "2.1";
-				}
-                else if(sAttribute->compare("http://earth.google.com/kml/2.0") == 0)
+                    ((KML *)pUserData)->validity = KML_VALIDITY_VALID;
+                    ((KML *)pUserData)->sVersion_ = "2.1";
+                }
+                else if(strcmp(ppszAttr[i + 1], "http://earth.google.com/kml/2.0") == 0)
                 {
-					((KML *)pUserData)->bValid_ = true;
-					((KML *)pUserData)->sVersion_ = "2.0";
-				}
-			}
-			delete sAttribute;
-		}
-	}
-	delete sName;
+                    ((KML *)pUserData)->validity = KML_VALIDITY_VALID;
+                    ((KML *)pUserData)->sVersion_ = "2.0";
+                }
+            }
+        }
+    }
 }
 
 void XMLCALL KML::endElement(void* pUserData, const char* pszName)
@@ -345,10 +363,13 @@ void XMLCALL KML::dataHandler(void* pUserData, const char* pszData, int nLen)
 
 bool KML::isValid()
 {
-	checkValidity();
-	
-    CPLDebug("KML", "Valid: %d Version: %s", bValid_, sVersion_.c_str());
-	return bValid_;
+    checkValidity();
+
+    if( validity == KML_VALIDITY_VALID )
+        CPLDebug("KML", "Valid: %d Version: %s", 
+                 validity == KML_VALIDITY_VALID, sVersion_.c_str());
+
+    return validity == KML_VALIDITY_VALID;
 }
 
 std::string KML::getError() const
