@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr2kmlgeometry.cpp 13358 2007-12-17 20:49:31Z rouault $
+ * $Id: ogr2kmlgeometry.cpp 14440 2008-05-10 21:43:42Z rouault $
  *
  * Project:  KML Driver
  * Purpose:  Implementation of OGR -> KML geometries writer.
@@ -374,7 +374,7 @@ CPLXMLNode* OGR_G_ExportEnvelopeToKMLTree( OGRGeometryH hGeometry )
     char* pszY = NULL;
 
     memset( &sEnvelope, 0, sizeof(sEnvelope) );
-    (static_cast<OGRGeometry*>(hGeometry))->getEnvelope( &sEnvelope );
+    ((OGRGeometry*)(hGeometry))->getEnvelope( &sEnvelope );
 
     if( sEnvelope.MinX == 0 && sEnvelope.MaxX == 0 
         && sEnvelope.MaxX == 0 && sEnvelope.MaxY == 0 )
