@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrshapedatasource.cpp 11928 2007-08-22 20:43:30Z mloskot $
+ * $Id: ogrshapedatasource.cpp 15328 2008-09-07 12:07:27Z rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRShapeDataSource class.
@@ -31,7 +31,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrshapedatasource.cpp 11928 2007-08-22 20:43:30Z mloskot $");
+CPL_CVSID("$Id: ogrshapedatasource.cpp 15328 2008-09-07 12:07:27Z rouault $");
 
 /************************************************************************/
 /*                         OGRShapeDataSource()                         */
@@ -481,7 +481,7 @@ OGRShapeDataSource::CreateLayer( const char * pszLayerName,
         CPLError( CE_Failure, CPLE_NotSupported,
                   "Geometry type of `%s' not supported in shapefiles.\n"
                   "Type can be overridden with a layer creation option\n"
-                  "of SHPT=POINT/ARC/POLYGON/MULTIPOINT.\n",
+                  "of SHPT=POINT/ARC/POLYGON/MULTIPOINT/POINTZ/ARCZ/POLYGONZ/MULTIPOINTZ.\n",
                   OGRGeometryTypeToName(eType) );
         return NULL;
     }
@@ -611,6 +611,8 @@ int OGRShapeDataSource::TestCapability( const char * pszCap )
 {
     if( EQUAL(pszCap,ODsCCreateLayer) )
         return TRUE;
+    else if( EQUAL(pszCap,ODsCDeleteLayer) )
+        return TRUE;
     else
         return FALSE;
 }
@@ -736,3 +738,43 @@ OGRLayer * OGRShapeDataSource::ExecuteSQL( const char *pszStatement,
     return NULL;
 }
 
+
+/************************************************************************/
+/*                            DeleteLayer()                             */
+/************************************************************************/
+
+OGRErr OGRShapeDataSource::DeleteLayer( int iLayer )
+
+{
+    char *pszFilename;
+
+    if( iLayer < 0 || iLayer >= nLayers )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Layer %d not in legal range of 0 to %d.", 
+                  iLayer, nLayers-1 );
+        return OGRERR_FAILURE;
+    }
+
+    pszFilename = CPLStrdup(((OGRShapeLayer*) papoLayers[iLayer])->GetFullName());
+
+    delete papoLayers[iLayer];
+
+    while( iLayer < nLayers - 1 )
+    {
+        papoLayers[iLayer] = papoLayers[iLayer+1];
+        iLayer++;
+    }
+
+    nLayers--;
+
+    VSIUnlink( CPLResetExtension(pszFilename, "shp") );
+    VSIUnlink( CPLResetExtension(pszFilename, "shx") );
+    VSIUnlink( CPLResetExtension(pszFilename, "dbf") );
+    VSIUnlink( CPLResetExtension(pszFilename, "prj") );
+    VSIUnlink( CPLResetExtension(pszFilename, "qix") );
+
+    CPLFree( pszFilename );
+
+    return OGRERR_NONE;
+}
