@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_string.h 13339 2007-12-14 18:10:05Z warmerdam $
+ * $Id: cpl_string.h 14851 2008-07-07 18:57:42Z mloskot $
  *
  * Name:     cpl_string.h
  * Project:  CPL - Common Portability Library
@@ -74,6 +74,8 @@ char CPL_DLL **CSLTokenizeString2( const char *pszString,
 #define CSLT_ALLOWEMPTYTOKENS   0x0002
 #define CSLT_PRESERVEQUOTES     0x0004
 #define CSLT_PRESERVEESCAPES    0x0008
+#define CSLT_STRIPLEADSPACES    0x0010
+#define CSLT_STRIPENDSPACES     0x0020
 
 int CPL_DLL CSLPrint(char **papszStrList, FILE *fpOut);
 char CPL_DLL **CSLLoad(const char *pszFname);
@@ -88,12 +90,13 @@ char CPL_DLL **CSLRemoveStrings(char **papszStrList, int nFirstLineToDelete,
 int CPL_DLL CSLFindString( char **, const char * );
 int CPL_DLL CSLPartialFindString( char **papszHaystack, 
 	const char * pszNeedle );
+int CPL_DLL CSLFindName(char **papszStrList, const char *pszName);
 int CPL_DLL CSLTestBoolean( const char *pszValue );
 int CPL_DLL CSLFetchBoolean( char **papszStrList, const char *pszKey, 
                              int bDefault );
 
-const char CPL_DLL *CPLSPrintf(const char *fmt, ...);
-char CPL_DLL **CSLAppendPrintf(char **papszStrList, char *fmt, ...);
+const char CPL_DLL *CPLSPrintf(const char *fmt, ...) CPL_PRINT_FUNC_FORMAT(1, 2);
+char CPL_DLL **CSLAppendPrintf(char **papszStrList, const char *fmt, ...) CPL_PRINT_FUNC_FORMAT(2, 3);
 int CPL_DLL CPLVASPrintf(char **buf, const char *fmt, va_list args );
 
 const char CPL_DLL *
@@ -125,6 +128,36 @@ char CPL_DLL *CPLUnescapeString( const char *pszString, int *pnLength,
 char CPL_DLL *CPLBinaryToHex( int nBytes, const GByte *pabyData );
 GByte CPL_DLL *CPLHexToBinary( const char *pszHex, int *pnBytes );
 
+typedef enum
+{
+    CPL_VALUE_STRING,
+    CPL_VALUE_REAL,
+    CPL_VALUE_INTEGER
+} CPLValueType;
+
+CPLValueType CPL_DLL CPLGetValueType(const char* pszValue);
+
+/* -------------------------------------------------------------------- */
+/*      RFC 23 character set conversion/recoding API (cpl_recode.cpp).  */
+/* -------------------------------------------------------------------- */
+#define CPL_ENC_LOCALE     ""
+#define CPL_ENC_UTF8       "UTF-8"
+#define CPL_ENC_UTF16      "UTF-16"
+#define CPL_ENC_UCS2       "UCS-2"
+#define CPL_ENC_UCS4       "UCS-4"
+#define CPL_ENC_ASCII      "ASCII"
+#define CPL_ENC_ISO8859_1  "ISO-8859-1"
+
+char CPL_DLL *CPLRecode( const char *pszSource, 
+                         const char *pszSrcEncoding, 
+                         const char *pszDstEncoding );
+char CPL_DLL *CPLRecodeFromWChar( const wchar_t *pwszSource, 
+                                  const char *pszSrcEncoding, 
+                                  const char *pszDstEncoding );
+wchar_t CPL_DLL *CPLRecodeToWChar( const char *pszSource,
+                                   const char *pszSrcEncoding, 
+                                   const char *pszDstEncoding );
+
 CPL_C_END
 
 /************************************************************************/
@@ -148,10 +181,11 @@ CPL_C_END
  * 1200 - VC++ 6.0
  * 1200-1202 - eVC++ 4.0
  */
-#if (_MSC_VER <= 1202)
-#  define MSVC_OLD_STUPID_BEHAVIOUR
+#if defined(_MSC_VER) 
+# if (_MSC_VER <= 1202) 
+#  define MSVC_OLD_STUPID_BEHAVIOUR 
+# endif
 #endif
- 
 
 /* Avoid C2614 errors */
 #ifdef MSVC_OLD_STUPID_BEHAVIOUR
@@ -202,7 +236,8 @@ public:
 
     void Clear() { resize(0); }
 
-    CPLString &Printf( const char *pszFormat, ... );
+    /* There seems to be a bug in the way the compiler count indices... Should be CPL_PRINT_FUNC_FORMAT (1, 2) */
+    CPLString &Printf( const char *pszFormat, ... ) CPL_PRINT_FUNC_FORMAT (2, 3);
     CPLString &vPrintf( const char *pszFormat, va_list args );
     CPLString &FormatC( double dfValue, const char *pszFormat = NULL );
     CPLString &Trim();
